"""This file contains the main function that's called by the CLI of the library.
"""

import os
import sys
import time

import numpy as np

from discovery import discover_benchmark_modules, get_benchmark_functions
from stats import has_enough_measurements


def main(top_level_path, stop_on_error):
    """Top level function called when the CLI is invoked.
    """
    if "::" in top_level_path:
        if top_level_path.count("::") > 1:
            raise AssertionError(f"Invalid path {top_level_path}")
        top_level_path, benchmark_function_name = top_level_path.split("::")
    else:
        benchmark_function_name = None

    if not os.path.exists(top_level_path):
        raise AssertionError(
            f"The top-level path {top_level_path} doesn't exist"
        )

    modules = [module for module in discover_benchmark_modules(top_level_path)]
    benchmark_dicts = []
    for module in modules:
        benchmark_functions = [
            function for function in
            get_benchmark_functions(module, benchmark_function_name)
        ]
        for benchmark_function in benchmark_functions:
            try:
                compiler, runner = benchmark_function()
            except (TypeError, ValueError):
                error_message = (
                    f"benchmark_function '{benchmark_function.__name__}'"
                    f" must return a two tuple value (compiler, runner)."
                )
                if stop_on_error is False:
                    print(error_message, file=sys.stderr)
                    continue
                else:
                    raise AssertionError(error_message)
            measurements_ns = np.array([])
            if compiler:
                start_compile_time_s = time.time()
                try:
                    compiled_callable = compiler()
                except Exception as e:
                    error_message = (
                        f"Compilation of {benchmark_function.__name__} failed"
                        f" because of {e}"
                    )
                    if stop_on_error is False:
                        print(error_message, file=sys.stderr)
                        continue
                    else:
                        raise AssertionError(error_message)
                total_compile_time_s = time.time() - start_compile_time_s
                runner_args = (compiled_callable,)
            else:
                total_compile_time_s = 0
                runner_args = ()
            while not has_enough_measurements(measurements_ns):
                try:
                    measurement_ns = runner(*runner_args)
                except Exception as e:
                    error_message = (
                        f"Runner of {benchmark_function.__name__} failed"
                        f" because of {e}"
                    )
                    if stop_on_error is False:
                        print(error_message, file=sys.stderr)
                        # Recover from runner error by breaking out of this loop
                        # and continuing forward.
                        break
                    else:
                        raise AssertionError(error_message)
                if not isinstance(measurement_ns, int):
                    error_message = (
                        f"Expected benchmark runner function"
                        f" to return an int, got {measurement_ns}"
                    )
                    if stop_on_error is False:
                        print(error_message, file=sys.stderr)
                        continue
                    else:
                        raise AssertionError(error_message)
                measurements_ns = np.append(measurements_ns, measurement_ns)

            if len(measurements_ns) > 0:
                measurements_s = [t * 1e-9 for t in measurements_ns]
                benchmark_identifier = ":".join([
                    module.__name__,
                    benchmark_function.__name__
                ])
                benchmark_dicts.append(
                    {
                        "name": benchmark_identifier,
                        "compile_time": total_compile_time_s,
                        "execution_time": list(measurements_s),
                    }
                )

    return benchmark_dicts



"""Test that anonymous structs/unions are transparent to member access"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AnonymousTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @skipIf(
        compiler="icc",
        bugnumber="llvm.org/pr15036: LLDB generates an incorrect AST layout for an anonymous struct when DWARF is generated by ICC")
    def test_expr_nest(self):
        self.build()
        self.common_setup(self.line0)

        # These should display correctly.
        self.expect("expression n->foo.d", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=["= 4"])

        self.expect("expression n->b", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=["= 2"])

    def test_expr_child(self):
        self.build()
        self.common_setup(self.line1)

        # These should display correctly.
        self.expect("expression c->foo.d", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=["= 4"])

        self.expect(
            "expression c->grandchild.b",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["= 2"])

    @skipIf(
        compiler="icc",
        bugnumber="llvm.org/pr15036: This particular regression was introduced by r181498")
    def test_expr_grandchild(self):
        self.build()
        self.common_setup(self.line2)

        # These should display correctly.
        self.expect("expression g.child.foo.d", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=["= 4"])

        self.expect("expression g.child.b", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=["= 2"])

    def test_expr_parent(self):
        self.build()
        if "clang" in self.getCompiler() and "3.4" in self.getCompilerVersion():
            self.skipTest(
                "llvm.org/pr16214 -- clang emits partial DWARF for structures referenced via typedef")
        self.common_setup(self.line2)

        # These should display correctly.
        self.expect("expression pz", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=["(type_z *) $", " = NULL"])

        self.expect("expression z.y", VARIABLES_DISPLAYED_CORRECTLY,
                    substrs=["(type_y) $", "dummy = 2"])

    def test_expr_null(self):
        self.build()
        self.common_setup(self.line2)

        # This should fail because pz is 0, but it succeeds on OS/X.
        # This fails on Linux with an upstream error "Couldn't dematerialize struct", as does "p *n" with "int *n = 0".
        # Note that this can also trigger llvm.org/pr15036 when run
        # interactively at the lldb command prompt.
        self.expect("expression *(type_z *)pz", error=True)

    def test_child_by_name(self):
        self.build()

        # Set debugger into synchronous mode
        self.dbg.SetAsync(False)

        # Create a target
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        break_in_main = target.BreakpointCreateBySourceRegex(
            '// Set breakpoint 2 here.', lldb.SBFileSpec(self.source))
        self.assertTrue(break_in_main, VALID_BREAKPOINT)

        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, break_in_main)
        if len(threads) != 1:
            self.fail("Failed to stop at breakpoint in main.")

        thread = threads[0]
        frame = thread.frames[0]

        if not frame.IsValid():
            self.fail("Failed to get frame 0.")

        var_n = frame.FindVariable("n")
        if not var_n.IsValid():
            self.fail("Failed to get the variable 'n'")

        elem_a = var_n.GetChildMemberWithName("a")
        if not elem_a.IsValid():
            self.fail("Failed to get the element a in n")

        error = lldb.SBError()
        value = elem_a.GetValueAsSigned(error, 1000)
        if not error.Success() or value != 0:
            self.fail("failed to get the correct value for element a in n")

    def test_nest_flat(self):
        self.build()
        self.common_setup(self.line2)

        # These should display correctly.
        self.expect('frame variable n --flat',
                    substrs=['n.a = 0',
                             'n.b = 2',
                             'n.foo.c = 0',
                             'n.foo.d = 4'])

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break in main.c.
        self.source = 'main.c'
        self.line0 = line_number(self.source, '// Set breakpoint 0 here.')
        self.line1 = line_number(self.source, '// Set breakpoint 1 here.')
        self.line2 = line_number(self.source, '// Set breakpoint 2 here.')

    def common_setup(self, line):

        # Set debugger into synchronous mode
        self.dbg.SetAsync(False)

        # Create a target
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set breakpoints inside and outside methods that take pointers to the
        # containing struct.
        lldbutil.run_break_set_by_file_and_line(
            self, self.source, line, num_expected_locations=1, loc_exact=True)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno = 1, expected_hit_count = 1)



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetCString()
    obj.Fail()
    obj.Success()
    obj.GetError()
    obj.GetType()
    obj.SetError(5, lldb.eErrorTypeGeneric)
    obj.SetErrorToErrno()
    obj.SetErrorToGenericError()
    obj.SetErrorString("xyz")
    obj.SetErrorString(None)
    obj.SetErrorStringWithFormat("%s!", "error")
    obj.SetErrorStringWithFormat(None)
    obj.SetErrorStringWithFormat("error")
    obj.SetErrorStringWithFormat("%s %s", "warning", "danger")
    obj.SetErrorStringWithFormat("%s %s %s", "danger", "will", "robinson")
    obj.GetDescription(lldb.SBStream())
    obj.Clear()



# On macOS, system python binaries like /usr/bin/python and $(xcrun -f python3)
# are shims. They do some light validation work and then spawn the "real" python
# binary. Find the "real" python by asking dyld -- sys.executable reports the
# wrong thing more often than not. This is also useful when we're running under
# a Homebrew python3 binary, which also appears to be some kind of shim.
def getDarwinRealPythonExecutable():
    import ctypes
    dyld = ctypes.cdll.LoadLibrary('/usr/lib/system/libdyld.dylib')
    namelen = ctypes.c_ulong(1024)
    name = ctypes.create_string_buffer(b'\000', namelen.value)
    dyld._NSGetExecutablePath(ctypes.byref(name), ctypes.byref(namelen))
    return name.value.decode('utf-8').strip()

print(getDarwinRealPythonExecutable())



"""Test that SBFrame::FindValue finds things but does not duplicate the entire variables list"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SBFrameFindValueTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)
    NO_DEBUG_INFO_TESTCASE = True

    def test_formatters_api(self):
        """Test that SBFrame::FindValue finds things but does not duplicate the entire variables list"""
        self.build()
        self.setTearDownCleanup()

        exe = self.getBuildArtifact("a.out")

        # Create the target
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set the breakpoints
        breakpoint = target.BreakpointCreateBySourceRegex(
            'Set breakpoint here', lldb.SBFileSpec("main.cpp"))
        self.assertTrue(breakpoint.GetNumLocations() > 0, VALID_BREAKPOINT)

        # Launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # Frame #0 should be at our breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, breakpoint)

        self.assertEquals(len(threads), 1)
        self.thread = threads[0]
        self.frame = self.thread.frames[0]
        self.assertTrue(self.frame, "Frame 0 is valid.")

        self.assertTrue(
            self.frame.GetVariables(
                True,
                True,
                False,
                True).GetSize() == 2,
            "variable count is off")
        self.assertFalse(
            self.frame.FindValue(
                "NoSuchThing",
                lldb.eValueTypeVariableArgument,
                lldb.eDynamicCanRunTarget).IsValid(),
            "found something that should not be here")
        self.assertTrue(
            self.frame.GetVariables(
                True,
                True,
                False,
                True).GetSize() == 2,
            "variable count is off after failed FindValue()")
        self.assertTrue(
            self.frame.FindValue(
                "a",
                lldb.eValueTypeVariableArgument,
                lldb.eDynamicCanRunTarget).IsValid(),
            "FindValue() didn't find an argument")
        self.assertTrue(
            self.frame.GetVariables(
                True,
                True,
                False,
                True).GetSize() == 2,
            "variable count is off after successful FindValue()")



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetName()
    obj.GetMangledName()
    obj.GetInstructions(lldb.SBTarget())
    obj.GetStartAddress()
    obj.GetEndAddress()
    obj.GetPrologueByteSize()
    obj.GetType()
    obj.GetDescription(lldb.SBStream())



"""
Test more expression command sequences with objective-c.
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FoundationTestCaseNSArray(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def test_NSArray_expr_commands(self):
        """Test expression commands for NSArray."""
        self.build()
        self.target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
                self, '// Break here for NSArray tests',
                lldb.SBFileSpec('main.m', False))

        self.runCmd("thread backtrace")
        self.expect("expression (int)[nil_mutable_array count]",
                    patterns=["\(int\) \$.* = 0"])
        self.expect("expression (int)[array1 count]",
                    patterns=["\(int\) \$.* = 3"])
        self.expect("expression (int)[array2 count]",
                    patterns=["\(int\) \$.* = 3"])
        self.expect("expression (int)array1.count",
                    patterns=["\(int\) \$.* = 3"])
        self.expect("expression (int)array2.count",
                    patterns=["\(int\) \$.* = 3"])
        self.runCmd("process continue")



"""
Test lldb data formatter subsystem.
"""



import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class ScriptDataFormatterTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def test_with_run_command(self):
        """Test data formatter commands."""
        self.build()
        self.data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number('main.cpp', '// Set break point at this line.')

    def data_formatter_commands(self):
        """Test that that file and class static variables display correctly."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True)

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd('type format clear', check=False)
            self.runCmd('type summary clear', check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Set the script here to ease the formatting
        script = 'a = valobj.GetChildMemberWithName(\'integer\'); a_val = a.GetValue(); str = \'Hello from Python, \' + a_val + \' time\'; return str + (\'!\' if a_val == \'1\' else \'s!\');'

        self.runCmd(
            "type summary add i_am_cool --python-script \"%s\"" %
            script)
        self.expect('type summary list i_am_cool', substrs=[script])

        self.expect("frame variable one",
                    substrs=['Hello from Python',
                             '1 time!'])

        self.expect("frame variable two",
                    substrs=['Hello from Python',
                             '4 times!'])

        self.runCmd("n")  # skip ahead to make values change

        self.expect("frame variable three",
                    substrs=['Hello from Python, 10 times!',
                             'Hello from Python, 4 times!'])

        self.runCmd("n")  # skip ahead to make values change

        self.expect("frame variable two",
                    substrs=['Hello from Python',
                             '1 time!'])

        script = 'a = valobj.GetChildMemberWithName(\'integer\'); a_val = a.GetValue(); str = \'int says \' + a_val; return str;'

        # Check that changes in the script are immediately reflected
        self.runCmd(
            "type summary add i_am_cool --python-script \"%s\"" %
            script)

        self.expect("frame variable two",
                    substrs=['int says 1'])

        self.expect("frame variable twoptr",
                    substrs=['int says 1'])

        # Change the summary
        self.runCmd(
            "type summary add --summary-string \"int says ${var.integer}, and float says ${var.floating}\" i_am_cool")

        self.expect("frame variable two",
                    substrs=['int says 1',
                             'and float says 2.71'])
        # Try it for pointers
        self.expect("frame variable twoptr",
                    substrs=['int says 1',
                             'and float says 2.71'])

        # Force a failure for pointers
        self.runCmd(
            "type summary add i_am_cool -p --python-script \"%s\"" %
            script)

        self.expect("frame variable twoptr", matching=False,
                    substrs=['and float says 2.71'])

        script = 'return \'Python summary\''

        self.runCmd(
            "type summary add --name test_summary --python-script \"%s\"" %
            script)

        # attach the Python named summary to someone
        self.expect("frame variable one --summary test_summary",
                    substrs=['Python summary'])

        # should not bind to the type
        self.expect("frame variable two", matching=False,
                    substrs=['Python summary'])

        # and should not stick to the variable
        self.expect("frame variable one", matching=False,
                    substrs=['Python summary'])

        self.runCmd(
            "type summary add i_am_cool --summary-string \"Text summary\"")

        # should be temporary only
        self.expect("frame variable one", matching=False,
                    substrs=['Python summary'])

        # use the type summary
        self.expect("frame variable two",
                    substrs=['Text summary'])

        self.runCmd("n")  # skip ahead to make values change

        # both should use the type summary now
        self.expect("frame variable one",
                    substrs=['Text summary'])

        self.expect("frame variable two",
                    substrs=['Text summary'])

        # disable type summary for pointers, and make a Python regex summary
        self.runCmd(
            "type summary add i_am_cool -p --summary-string \"Text summary\"")
        self.runCmd("type summary add -x cool --python-script \"%s\"" % script)

        # variables should stick to the type summary
        self.expect("frame variable one",
                    substrs=['Text summary'])

        self.expect("frame variable two",
                    substrs=['Text summary'])

        # array and pointer should match the Python one
        self.expect("frame variable twoptr",
                    substrs=['Python summary'])

        self.expect("frame variable array",
                    substrs=['Python summary'])

        # return pointers to the type summary
        self.runCmd(
            "type summary add i_am_cool --summary-string \"Text summary\"")

        self.expect("frame variable one",
                    substrs=['Text summary'])

        self.expect("frame variable two",
                    substrs=['Text summary'])

        self.expect("frame variable twoptr",
                    substrs=['Text summary'])

        self.expect("frame variable array",
                    substrs=['Python summary'])



"""
Test that variables of floating point types are displayed correctly.
"""



import AbstractBase

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FloatTypesTestCase(AbstractBase.GenericTester):

    mydir = AbstractBase.GenericTester.compute_mydir(__file__)

    def test_float_type(self):
        """Test that float-type variables are displayed correctly."""
        self.build_and_run('float.cpp', set(['float']))

    @skipUnlessDarwin
    def test_float_type_from_block(self):
        """Test that float-type variables are displayed correctly from a block."""
        self.build_and_run('float.cpp', set(['float']), bc=True)



import lldb
import binascii
import os.path
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestGDBRemoteClient(GDBRemoteTestBase):

    mydir = TestBase.compute_mydir(__file__)

    class gPacketResponder(MockGDBServerResponder):
        registers = [
            "name:rax;bitsize:64;offset:0;encoding:uint;format:hex;set:General Purpose Registers;ehframe:0;dwarf:0;",
            "name:rbx;bitsize:64;offset:8;encoding:uint;format:hex;set:General Purpose Registers;ehframe:3;dwarf:3;",
            "name:rcx;bitsize:64;offset:16;encoding:uint;format:hex;set:General Purpose Registers;ehframe:2;dwarf:2;generic:arg4;",
            "name:rdx;bitsize:64;offset:24;encoding:uint;format:hex;set:General Purpose Registers;ehframe:1;dwarf:1;generic:arg3;",
            "name:rdi;bitsize:64;offset:32;encoding:uint;format:hex;set:General Purpose Registers;ehframe:5;dwarf:5;generic:arg1;",
            "name:rsi;bitsize:64;offset:40;encoding:uint;format:hex;set:General Purpose Registers;ehframe:4;dwarf:4;generic:arg2;",
            "name:rbp;bitsize:64;offset:48;encoding:uint;format:hex;set:General Purpose Registers;ehframe:6;dwarf:6;generic:fp;",
            "name:rsp;bitsize:64;offset:56;encoding:uint;format:hex;set:General Purpose Registers;ehframe:7;dwarf:7;generic:sp;",
        ]

        def qRegisterInfo(self, num):
            try:
                return self.registers[num]
            except IndexError:
                return "E45"

        def readRegisters(self):
            return len(self.registers) * 16 * '0'

        def readRegister(self, register):
            return "0000000000000000"

    def test_connect(self):
        """Test connecting to a remote gdb server"""
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        self.assertPacketLogContains(["qProcessInfo", "qfThreadInfo"])

    def test_attach_fail(self):
        error_msg = "mock-error-msg"

        class MyResponder(MockGDBServerResponder):
            # Pretend we don't have any process during the initial queries.
            def qC(self):
                return "E42"

            def qfThreadInfo(self):
                return "OK" # No threads.

            # Then, when we are asked to attach, error out.
            def vAttach(self, pid):
                return "E42;" + binascii.hexlify(error_msg.encode()).decode()

        self.server.responder = MyResponder()

        target = self.dbg.CreateTarget("")
        process = self.connect(target)
        lldbutil.expect_state_changes(self, self.dbg.GetListener(), process, [lldb.eStateConnected])

        error = lldb.SBError()
        target.AttachToProcessWithID(lldb.SBListener(), 47, error)
        self.assertEquals(error_msg, error.GetCString())

    def test_launch_fail(self):
        class MyResponder(MockGDBServerResponder):
            # Pretend we don't have any process during the initial queries.
            def qC(self):
                return "E42"

            def qfThreadInfo(self):
                return "OK" # No threads.

            # Then, when we are asked to attach, error out.
            def A(self, packet):
                return "E47"

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(self, self.dbg.GetListener(), process, [lldb.eStateConnected])

        error = lldb.SBError()
        target.Launch(lldb.SBListener(), None, None, None, None, None,
                None, 0, True, error)
        self.assertEquals("'A' packet returned an error: 71", error.GetCString())

    def test_read_registers_using_g_packets(self):
        """Test reading registers using 'g' packets (default behavior)"""
        self.dbg.HandleCommand(
                "settings set plugin.process.gdb-remote.use-g-packet-for-reading true")
        self.addTearDownHook(lambda:
                self.runCmd("settings set plugin.process.gdb-remote.use-g-packet-for-reading false"))
        self.server.responder = self.gPacketResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.assertEquals(1, self.server.responder.packetLog.count("g"))
        self.server.responder.packetLog = []
        self.read_registers(process)
        # Reading registers should not cause any 'p' packets to be exchanged.
        self.assertEquals(
                0, len([p for p in self.server.responder.packetLog if p.startswith("p")]))

    def test_read_registers_using_p_packets(self):
        """Test reading registers using 'p' packets"""
        self.dbg.HandleCommand(
                "settings set plugin.process.gdb-remote.use-g-packet-for-reading false")
        self.server.responder = self.gPacketResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.read_registers(process)
        self.assertNotIn("g", self.server.responder.packetLog)
        self.assertGreater(
                len([p for p in self.server.responder.packetLog if p.startswith("p")]), 0)

    def test_write_registers_using_P_packets(self):
        """Test writing registers using 'P' packets (default behavior)"""
        self.server.responder = self.gPacketResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.write_registers(process)
        self.assertEquals(0, len(
                [p for p in self.server.responder.packetLog if p.startswith("G")]))
        self.assertGreater(
                len([p for p in self.server.responder.packetLog if p.startswith("P")]), 0)

    def test_write_registers_using_G_packets(self):
        """Test writing registers using 'G' packets"""

        class MyResponder(self.gPacketResponder):
            def readRegister(self, register):
                # empty string means unsupported
                return ""

        self.server.responder = MyResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.write_registers(process)
        self.assertEquals(0, len(
                [p for p in self.server.responder.packetLog if p.startswith("P")]))
        self.assertGreater(len(
                [p for p in self.server.responder.packetLog if p.startswith("G")]), 0)

    def read_registers(self, process):
        self.for_each_gpr(
                process, lambda r: self.assertEquals("0x0000000000000000", r.GetValue()))

    def write_registers(self, process):
        self.for_each_gpr(
                process, lambda r: r.SetValueFromCString("0x0000000000000000"))

    def for_each_gpr(self, process, operation):
        registers = process.GetThreadAtIndex(0).GetFrameAtIndex(0).GetRegisters()
        self.assertGreater(registers.GetSize(), 0)
        regSet = registers[0]
        numChildren = regSet.GetNumChildren()
        self.assertGreater(numChildren, 0)
        for i in range(numChildren):
            operation(regSet.GetChildAtIndex(i))

    def test_launch_A(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self, *args, **kwargs):
                self.started = False
                return super().__init__(*args, **kwargs)

            def qC(self):
                if self.started:
                    return "QCp10.10"
                else:
                    return "E42"

            def qfThreadInfo(self):
                if self.started:
                    return "mp10.10"
                else:
                   return "E42"

            def qsThreadInfo(self):
                return "l"

            def A(self, packet):
                self.started = True
                return "OK"

            def qLaunchSuccess(self):
                if self.started:
                    return "OK"
                return "E42"

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        # NB: apparently GDB packets are using "/" on Windows too
        exe_path = self.getBuildArtifact("a").replace(os.path.sep, '/')
        exe_hex = binascii.b2a_hex(exe_path.encode()).decode()
        process = self.connect(target)
        lldbutil.expect_state_changes(self, self.dbg.GetListener(), process,
                                      [lldb.eStateConnected])

        target.Launch(lldb.SBListener(),
                      ["arg1", "arg2", "arg3"],  # argv
                      [],  # envp
                      None,  # stdin_path
                      None,  # stdout_path
                      None,  # stderr_path
                      None,  # working_directory
                      0,  # launch_flags
                      True,  # stop_at_entry
                      lldb.SBError())  # error
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetProcessID(), 16)

        self.assertPacketLogContains([
          "A%d,0,%s,8,1,61726731,8,2,61726732,8,3,61726733" % (
              len(exe_hex), exe_hex),
        ])

    def test_launch_vRun(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self, *args, **kwargs):
                self.started = False
                return super().__init__(*args, **kwargs)

            def qC(self):
                if self.started:
                    return "QCp10.10"
                else:
                    return "E42"

            def qfThreadInfo(self):
                if self.started:
                    return "mp10.10"
                else:
                   return "E42"

            def qsThreadInfo(self):
                return "l"

            def vRun(self, packet):
                self.started = True
                return "T13"

            def A(self, packet):
                return "E28"

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        # NB: apparently GDB packets are using "/" on Windows too
        exe_path = self.getBuildArtifact("a").replace(os.path.sep, '/')
        exe_hex = binascii.b2a_hex(exe_path.encode()).decode()
        process = self.connect(target)
        lldbutil.expect_state_changes(self, self.dbg.GetListener(), process,
                                      [lldb.eStateConnected])

        process = target.Launch(lldb.SBListener(),
                                ["arg1", "arg2", "arg3"],  # argv
                                [],  # envp
                                None,  # stdin_path
                                None,  # stdout_path
                                None,  # stderr_path
                                None,  # working_directory
                                0,  # launch_flags
                                True,  # stop_at_entry
                                lldb.SBError())  # error
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetProcessID(), 16)

        self.assertPacketLogContains([
          "vRun;%s;61726731;61726732;61726733" % (exe_hex,)
        ])

    def test_launch_QEnvironment(self):
        class MyResponder(MockGDBServerResponder):
            def qC(self):
                return "E42"

            def qfThreadInfo(self):
               return "E42"

            def vRun(self, packet):
                self.started = True
                return "E28"

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(self, self.dbg.GetListener(), process,
                                      [lldb.eStateConnected])

        target.Launch(lldb.SBListener(),
                      [],  # argv
                      ["PLAIN=foo",
                       "NEEDSENC=frob$",
                       "NEEDSENC2=fr*ob",
                       "NEEDSENC3=fro}b",
                       "NEEDSENC4=f#rob",
                       "EQUALS=foo=bar",
                       ],  # envp
                      None,  # stdin_path
                      None,  # stdout_path
                      None,  # stderr_path
                      None,  # working_directory
                      0,  # launch_flags
                      True,  # stop_at_entry
                      lldb.SBError())  # error

        self.assertPacketLogContains([
          "QEnvironment:PLAIN=foo",
          "QEnvironmentHexEncoded:4e45454453454e433d66726f6224",
          "QEnvironmentHexEncoded:4e45454453454e43323d66722a6f62",
          "QEnvironmentHexEncoded:4e45454453454e43333d66726f7d62",
          "QEnvironmentHexEncoded:4e45454453454e43343d6623726f62",
          "QEnvironment:EQUALS=foo=bar",
        ])

    def test_launch_QEnvironmentHexEncoded_only(self):
        class MyResponder(MockGDBServerResponder):
            def qC(self):
                return "E42"

            def qfThreadInfo(self):
               return "E42"

            def vRun(self, packet):
                self.started = True
                return "E28"

            def QEnvironment(self, packet):
                return ""

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(self, self.dbg.GetListener(), process,
                                      [lldb.eStateConnected])

        target.Launch(lldb.SBListener(),
                      [],  # argv
                      ["PLAIN=foo",
                       "NEEDSENC=frob$",
                       "NEEDSENC2=fr*ob",
                       "NEEDSENC3=fro}b",
                       "NEEDSENC4=f#rob",
                       "EQUALS=foo=bar",
                       ],  # envp
                      None,  # stdin_path
                      None,  # stdout_path
                      None,  # stderr_path
                      None,  # working_directory
                      0,  # launch_flags
                      True,  # stop_at_entry
                      lldb.SBError())  # error

        self.assertPacketLogContains([
          "QEnvironmentHexEncoded:504c41494e3d666f6f",
          "QEnvironmentHexEncoded:4e45454453454e433d66726f6224",
          "QEnvironmentHexEncoded:4e45454453454e43323d66722a6f62",
          "QEnvironmentHexEncoded:4e45454453454e43333d66726f7d62",
          "QEnvironmentHexEncoded:4e45454453454e43343d6623726f62",
          "QEnvironmentHexEncoded:455155414c533d666f6f3d626172",
        ])

    def test_detach_no_multiprocess(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self):
                super().__init__()
                self.detached = None

            def qfThreadInfo(self):
                return "10200"

            def D(self, packet):
                self.detached = packet
                return "OK"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget('')
        process = self.connect(target)
        process.Detach()
        self.assertEqual(self.server.responder.detached, "D")

    def test_detach_pid(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self, test_case):
                super().__init__()
                self.test_case = test_case
                self.detached = None

            def qSupported(self, client_supported):
                self.test_case.assertIn("multiprocess+", client_supported)
                return "multiprocess+;" + super().qSupported(client_supported)

            def qfThreadInfo(self):
                return "mp400.10200"

            def D(self, packet):
                self.detached = packet
                return "OK"

        self.server.responder = MyResponder(self)
        target = self.dbg.CreateTarget('')
        process = self.connect(target)
        process.Detach()
        self.assertRegex(self.server.responder.detached, r"D;0*400")

    def test_signal_gdb(self):
        class MyResponder(MockGDBServerResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+"

            def haltReason(self):
                return "S0a"

            def cont(self):
                return self.haltReason()

        self.server.responder = MyResponder()

        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.assertEqual(process.threads[0].GetStopReason(),
                         lldb.eStopReasonSignal)
        self.assertEqual(process.threads[0].GetStopDescription(100),
                         'signal SIGBUS')

    def test_signal_lldb_old(self):
        class MyResponder(MockGDBServerResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+"

            def qHostInfo(self):
                return "triple:61726d76372d756e6b6e6f776e2d6c696e75782d676e75;"

            def QThreadSuffixSupported(self):
                return "OK"

            def haltReason(self):
                return "S0a"

            def cont(self):
                return self.haltReason()

        self.server.responder = MyResponder()

        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.assertEqual(process.threads[0].GetStopReason(),
                         lldb.eStopReasonSignal)
        self.assertEqual(process.threads[0].GetStopDescription(100),
                         'signal SIGUSR1')

    def test_signal_lldb(self):
        class MyResponder(MockGDBServerResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+;native-signals+"

            def qHostInfo(self):
                return "triple:61726d76372d756e6b6e6f776e2d6c696e75782d676e75;"

            def haltReason(self):
                return "S0a"

            def cont(self):
                return self.haltReason()

        self.server.responder = MyResponder()

        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.assertEqual(process.threads[0].GetStopReason(),
                         lldb.eStopReasonSignal)
        self.assertEqual(process.threads[0].GetStopDescription(100),
                         'signal SIGUSR1')

    def do_siginfo_test(self, platform, target_yaml, raw_data, expected):
        class MyResponder(MockGDBServerResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+;qXfer:siginfo:read+"

            def qXferRead(self, obj, annex, offset, length):
                if obj == "siginfo":
                    return raw_data, False
                else:
                    return None, False

            def haltReason(self):
                return "T02"

            def cont(self):
                return self.haltReason()

        self.server.responder = MyResponder()

        self.runCmd("platform select " + platform)
        target = self.createTarget(target_yaml)
        process = self.connect(target)

        siginfo = process.threads[0].GetSiginfo()
        self.assertTrue(siginfo.GetError().Success(), siginfo.GetError())

        for key, value in expected.items():
            self.assertEqual(siginfo.GetValueForExpressionPath("." + key)
                             .GetValueAsUnsigned(),
                             value)


    def test_siginfo_linux_amd64(self):
        data = (
          # si_signo         si_errno        si_code
            "\x11\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00"
          # __pad0           si_pid          si_uid
            "\x00\x00\x00\x00\xbf\xf7\x0b\x00\xe8\x03\x00\x00"
          # si_status
            "\x0c\x00\x00\x00" + "\x00" * 100)
        expected = {
            "si_signo": 17,  # SIGCHLD
            "si_errno": 0,
            "si_code": 1,  # CLD_EXITED
            "_sifields._sigchld.si_pid": 784319,
            "_sifields._sigchld.si_uid": 1000,
            "_sifields._sigchld.si_status": 12,
            "_sifields._sigchld.si_utime": 0,
            "_sifields._sigchld.si_stime": 0,
        }
        self.do_siginfo_test("remote-linux", "basic_eh_frame.yaml",
                             data, expected)

    def test_siginfo_linux_i386(self):
        data = (
          # si_signo         si_errno        si_code
            "\x11\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00"
          # si_pid           si_uid          si_status
            "\x49\x43\x07\x00\xe8\x03\x00\x00\x0c\x00\x00\x00"
            + "\x00" * 104)
        expected = {
            "si_signo": 17,  # SIGCHLD
            "si_errno": 0,
            "si_code": 1,  # CLD_EXITED
            "_sifields._sigchld.si_pid": 475977,
            "_sifields._sigchld.si_uid": 1000,
            "_sifields._sigchld.si_status": 12,
            "_sifields._sigchld.si_utime": 0,
            "_sifields._sigchld.si_stime": 0,
        }
        self.do_siginfo_test("remote-linux", "basic_eh_frame-i386.yaml",
                             data, expected)

    def test_siginfo_freebsd_amd64(self):
        data = (
          # si_signo         si_errno        si_code
            "\x0b\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00"
          # si_pid           si_uid          si_status
            "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
          # si_addr
            "\x76\x98\xba\xdc\xfe\x00\x00\x00"
          # si_status                        si_trapno
            "\x00\x00\x00\x00\x00\x00\x00\x00\x0c\x00\x00\x00"
            + "\x00" * 36)

        expected = {
            "si_signo": 11,  # SIGSEGV
            "si_errno": 0,
            "si_code": 1,  # SEGV_MAPERR
            "si_addr": 0xfedcba9876,
            "_reason._fault._trapno": 12,
        }
        self.do_siginfo_test("remote-freebsd", "basic_eh_frame.yaml",
                             data, expected)



#!/usr/bin/env python

import lldb
import struct


class OperatingSystemPlugIn(object):
    """Class that provides data for an instance of a LLDB 'OperatingSystemPython' plug-in class"""

    def __init__(self, process):
        '''Initialization needs a valid.SBProcess object.

        This plug-in will get created after a live process is valid and has stopped for the
        first time.'''
        self.process = None
        self.registers = None
        self.threads = None
        if isinstance(process, lldb.SBProcess) and process.IsValid():
            self.process = process
            self.threads = None  # Will be an dictionary containing info for each thread

    def get_target(self):
        # NOTE: Don't use "lldb.target" when trying to get your target as the "lldb.target"
        # tracks the current target in the LLDB command interpreter which isn't the
        # correct thing to use for this plug-in.
        return self.process.target

    def create_thread(self, tid, context):
        if tid == 0x444444444:
            thread_info = {
                'tid': tid,
                'name': 'four',
                'queue': 'queue4',
                'state': 'stopped',
                'stop_reason': 'none'}
            self.threads.append(thread_info)
            return thread_info
        return None

    def get_thread_info(self):
        if not self.threads:
            # The sample dictionary below shows the values that can be returned for a thread
            # tid => thread ID (mandatory)
            # name => thread name (optional key/value pair)
            # queue => thread dispatch queue name (optional key/value pair)
            # state => thred state (mandatory, set to 'stopped' for now)
            # stop_reason => thread stop reason. (mandatory, usually set to 'none')
            #  Possible values include:
            #   'breakpoint' if the thread is stopped at a breakpoint
            #   'none' thread is just stopped because the process is stopped
            #   'trace' the thread just single stepped
            #   The usual value for this while threads are in memory is 'none'
            # register_data_addr => the address of the register data in memory (optional key/value pair)
            #   Specifying this key/value pair for a thread will avoid a call to get_register_data()
            #   and can be used when your registers are in a thread context structure that is contiguous
            #   in memory. Don't specify this if your register layout in memory doesn't match the layout
            # described by the dictionary returned from a call to the
            # get_register_info() method.
            self.threads = [{'tid': 0x111111111,
                             'name': 'one',
                             'queue': 'queue1',
                             'state': 'stopped',
                             'stop_reason': 'breakpoint'},
                            {'tid': 0x222222222,
                             'name': 'two',
                             'queue': 'queue2',
                             'state': 'stopped',
                             'stop_reason': 'none'},
                            {'tid': 0x333333333,
                             'name': 'three',
                             'queue': 'queue3',
                             'state': 'stopped',
                             'stop_reason': 'trace',
                             'register_data_addr': 0x100000000}]
        return self.threads

    def get_register_info(self):
        if self.registers is None:
            self.registers = dict()
            triple = self.process.target.triple
            if triple:
                arch = triple.split('-')[0]
                if arch == 'x86_64':
                    self.registers['sets'] = ['GPR', 'FPU', 'EXC']
                    self.registers['registers'] = [
                        {'name': 'rax', 'bitsize': 64, 'offset': 0, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 0, 'dwarf': 0},
                        {'name': 'rbx', 'bitsize': 64, 'offset': 8, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 3, 'dwarf': 3},
                        {'name': 'rcx', 'bitsize': 64, 'offset': 16, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 2, 'dwarf': 2, 'generic': 'arg4', 'alt-name': 'arg4', },
                        {'name': 'rdx', 'bitsize': 64, 'offset': 24, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 1, 'dwarf': 1, 'generic': 'arg3', 'alt-name': 'arg3', },
                        {'name': 'rdi', 'bitsize': 64, 'offset': 32, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 5, 'dwarf': 5, 'generic': 'arg1', 'alt-name': 'arg1', },
                        {'name': 'rsi', 'bitsize': 64, 'offset': 40, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 4, 'dwarf': 4, 'generic': 'arg2', 'alt-name': 'arg2', },
                        {'name': 'rbp', 'bitsize': 64, 'offset': 48, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 6, 'dwarf': 6, 'generic': 'fp', 'alt-name': 'fp', },
                        {'name': 'rsp', 'bitsize': 64, 'offset': 56, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 7, 'dwarf': 7, 'generic': 'sp', 'alt-name': 'sp', },
                        {'name': 'r8', 'bitsize': 64, 'offset': 64, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 8, 'dwarf': 8, 'generic': 'arg5', 'alt-name': 'arg5', },
                        {'name': 'r9', 'bitsize': 64, 'offset': 72, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 9, 'dwarf': 9, 'generic': 'arg6', 'alt-name': 'arg6', },
                        {'name': 'r10', 'bitsize': 64, 'offset': 80, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 10, 'dwarf': 10},
                        {'name': 'r11', 'bitsize': 64, 'offset': 88, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 11, 'dwarf': 11},
                        {'name': 'r12', 'bitsize': 64, 'offset': 96, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 12, 'dwarf': 12},
                        {'name': 'r13', 'bitsize': 64, 'offset': 104, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 13, 'dwarf': 13},
                        {'name': 'r14', 'bitsize': 64, 'offset': 112, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 14, 'dwarf': 14},
                        {'name': 'r15', 'bitsize': 64, 'offset': 120, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 15, 'dwarf': 15},
                        {'name': 'rip', 'bitsize': 64, 'offset': 128, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'gcc': 16, 'dwarf': 16, 'generic': 'pc', 'alt-name': 'pc'},
                        {'name': 'rflags', 'bitsize': 64, 'offset': 136, 'encoding': 'uint', 'format': 'hex', 'set': 0, 'generic': 'flags', 'alt-name': 'flags'},
                        {'name': 'cs', 'bitsize': 64, 'offset': 144, 'encoding': 'uint', 'format': 'hex', 'set': 0},
                        {'name': 'fs', 'bitsize': 64, 'offset': 152, 'encoding': 'uint', 'format': 'hex', 'set': 0},
                        {'name': 'gs', 'bitsize': 64, 'offset': 160, 'encoding': 'uint', 'format': 'hex', 'set': 0},
                    ]
        return self.registers

    def get_register_data(self, tid):
        if tid == 0x111111111:
            return struct.pack(
                '21Q',
                1,
                2,
                3,
                4,
                5,
                6,
                7,
                8,
                9,
                10,
                11,
                12,
                13,
                14,
                15,
                16,
                17,
                18,
                19,
                20,
                21)
        elif tid == 0x222222222:
            return struct.pack(
                '21Q',
                11,
                12,
                13,
                14,
                15,
                16,
                17,
                18,
                19,
                110,
                111,
                112,
                113,
                114,
                115,
                116,
                117,
                118,
                119,
                120,
                121)
        elif tid == 0x333333333:
            return struct.pack(
                '21Q',
                21,
                22,
                23,
                24,
                25,
                26,
                27,
                28,
                29,
                210,
                211,
                212,
                213,
                214,
                215,
                216,
                217,
                218,
                219,
                220,
                221)
        elif tid == 0x444444444:
            return struct.pack(
                '21Q',
                31,
                32,
                33,
                34,
                35,
                36,
                37,
                38,
                39,
                310,
                311,
                312,
                313,
                314,
                315,
                316,
                317,
                318,
                319,
                320,
                321)
        else:
            return struct.pack(
                '21Q',
                41,
                42,
                43,
                44,
                45,
                46,
                47,
                48,
                49,
                410,
                411,
                412,
                413,
                414,
                415,
                416,
                417,
                418,
                419,
                420,
                421)
        return None



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

#  This file contains functions to process sparse tensor outputs.

import ctypes
import functools
import numpy as np


@functools.lru_cache()
def _get_c_shared_lib(lib_name: str):
  """Loads and returns the requested C shared library.

  Args:
    lib_name: A string representing the C shared library.

  Returns:
    The C shared library.

  Raises:
    OSError: If there is any problem in loading the shared library.
    ValueError:  If the shared library doesn't contain the needed routine.
  """
  # This raises OSError exception if there is any problem in loading the shared
  # library.
  c_lib = ctypes.CDLL(lib_name)

  try:
    c_lib.convertFromMLIRSparseTensor.restype = ctypes.c_void_p
  except Exception as e:
    raise ValueError('Missing function convertFromMLIRSparseTensor from '
                     f'the C shared library: {e} ') from e

  return c_lib


def sparse_tensor_to_coo_tensor(support_lib, sparse, dtype):
  """Converts a sparse tensor to COO-flavored format.

  Args:
     support_lib: A string for the supporting C shared library.
     sparse: A ctypes.pointer to the sparse tensor descriptor.
     dtype: The numpy data type for the tensor elements.

  Returns:
    A tuple that contains the following values:
    rank: An integer for the rank of the tensor.
    nse: An interger for the number of non-zero values in the tensor.
    shape: A 1D numpy array of integers, for the shape of the tensor.
    values: A 1D numpy array, for the non-zero values in the tensor.
    indices: A 2D numpy array of integers, representing the indices for the
      non-zero values in the tensor.

  Raises:
    OSError: If there is any problem in loading the shared library.
    ValueError:  If the shared library doesn't contain the needed routine.
  """
  c_lib = _get_c_shared_lib(support_lib)

  rank = ctypes.c_ulonglong(0)
  nse = ctypes.c_ulonglong(0)
  shape = ctypes.POINTER(ctypes.c_ulonglong)()
  values = ctypes.POINTER(np.ctypeslib.as_ctypes_type(dtype))()
  indices = ctypes.POINTER(ctypes.c_ulonglong)()
  c_lib.convertFromMLIRSparseTensor(sparse, ctypes.byref(rank),
                                    ctypes.byref(nse), ctypes.byref(shape),
                                    ctypes.byref(values), ctypes.byref(indices))
  # Convert the returned values to the corresponding numpy types.
  shape = np.ctypeslib.as_array(shape, shape=[rank.value])
  values = np.ctypeslib.as_array(values, shape=[nse.value])
  indices = np.ctypeslib.as_array(indices, shape=[nse.value, rank.value])
  return rank, nse, shape, values, indices



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestWithLimitDebugInfo(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @skipIf(debug_info=no_match(["dwarf"]))
    def test_limit_debug_info(self):
        self.build()

        src_file = os.path.join(self.getSourceDir(), "main.cpp")
        src_file_spec = lldb.SBFileSpec(src_file)
        self.assertTrue(src_file_spec.IsValid(), "breakpoint file")

        # Get the path of the executable
        exe_path = self.getBuildArtifact("a.out")

        # Load the executable
        target = self.dbg.CreateTarget(exe_path)
        self.assertTrue(target.IsValid(), VALID_TARGET)

        # Break on main function
        breakpoint = target.BreakpointCreateBySourceRegex(
            "break here", src_file_spec)
        self.assertTrue(
            breakpoint.IsValid() and breakpoint.GetNumLocations() >= 1,
            VALID_BREAKPOINT)

        # Launch the process
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())
        self.assertTrue(process.IsValid(), PROCESS_IS_VALID)

        # Get the thread of the process
        self.assertEqual(
            process.GetState(), lldb.eStateStopped,
            PROCESS_STOPPED)
        thread = lldbutil.get_stopped_thread(
            process, lldb.eStopReasonBreakpoint)
        thread.StepInto()

        # Get frame for current thread
        frame = thread.GetSelectedFrame()

        self.expect_expr("1", result_type="int", result_value="1")

        v2 = frame.EvaluateExpression("this")
        self.assertTrue(
            v2.IsValid(),
            "'expr this' results in a valid SBValue object")
        self.assertTrue(
            v2.GetError().Success(),
            "'expr this' succeeds without an error.")



"""
Test that global operators are found and evaluated.
"""
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCppGlobalOperators(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def prepare_executable_and_get_frame(self):
        self.build()

        # Get main source file
        src_file = "main.cpp"
        src_file_spec = lldb.SBFileSpec(src_file)
        self.assertTrue(src_file_spec.IsValid(), "Main source file")

        # Get the path of the executable
        exe_path = self.getBuildArtifact("a.out")

        # Load the executable
        target = self.dbg.CreateTarget(exe_path)
        self.assertTrue(target.IsValid(), VALID_TARGET)

        # Break on main function
        main_breakpoint = target.BreakpointCreateBySourceRegex(
            "// break here", src_file_spec)
        self.assertTrue(
            main_breakpoint.IsValid() and main_breakpoint.GetNumLocations() >= 1,
            VALID_BREAKPOINT)

        # Launch the process
        args = None
        env = None
        process = target.LaunchSimple(
            args, env, self.get_process_working_directory())
        self.assertTrue(process.IsValid(), PROCESS_IS_VALID)

        # Get the thread of the process
        self.assertEqual(
            process.GetState(), lldb.eStateStopped,
            PROCESS_STOPPED)
        thread = lldbutil.get_stopped_thread(
            process, lldb.eStopReasonBreakpoint)

        return thread.GetSelectedFrame()

    def test_equals_operator(self):
        frame = self.prepare_executable_and_get_frame()

        test_result = frame.EvaluateExpression("operator==(s1, s2)")
        self.assertTrue(
            test_result.IsValid() and test_result.GetValue() == "false",
            "operator==(s1, s2) = false")

        test_result = frame.EvaluateExpression("operator==(s1, s3)")
        self.assertTrue(
            test_result.IsValid() and test_result.GetValue() == "true",
            "operator==(s1, s3) = true")

        test_result = frame.EvaluateExpression("operator==(s2, s3)")
        self.assertTrue(
            test_result.IsValid() and test_result.GetValue() == "false",
            "operator==(s2, s3) = false")

    def do_new_test(self, frame, expr, expected_value_name):
        """Evaluate a new expression, and check its result"""

        expected_value = frame.FindValue(
            expected_value_name, lldb.eValueTypeVariableGlobal)
        self.assertTrue(expected_value.IsValid())

        expected_value_addr = expected_value.AddressOf()
        self.assertTrue(expected_value_addr.IsValid())

        got = frame.EvaluateExpression(expr)
        self.assertTrue(got.IsValid())
        self.assertEqual(
            got.GetValueAsUnsigned(),
            expected_value_addr.GetValueAsUnsigned())
        got_type = got.GetType()
        self.assertTrue(got_type.IsPointerType())
        self.assertEqual(got_type.GetPointeeType().GetName(), "Struct")

    def test_operator_new(self):
        frame = self.prepare_executable_and_get_frame()

        self.do_new_test(frame, "new Struct()", "global_new_buf")
        self.do_new_test(frame, "new(new_tag) Struct()", "tagged_new_buf")



"""
Test lldb data formatter subsystem.
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CompactVectorsFormattingTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number('main.cpp', '// Set break point at this line.')

    @skipUnlessDarwin
    def test_with_run_command(self):
        """Test that that file and class static variables display correctly."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True)

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd('type summary clear', check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.expect(
            'frame variable',
            substrs=[
                '(vFloat) valueFL = (1.25, 0, 0.25, 0)',
                '(vDouble) valueDL = (1.25, 2.25)',
                '(int16_t[8]) valueI16 = (1, 0, 4, 0, 0, 1, 0, 4)',
                '(int32_t[4]) valueI32 = (1, 0, 4, 0)',
                '(vUInt8) valueU8 = (0x01, 0x00, 0x04, 0x00, 0x00, 0x01, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)',
                '(vUInt16) valueU16 = (1, 0, 4, 0, 0, 1, 0, 4)',
                '(vUInt32) valueU32 = (1, 2, 3, 4)',
                '(vSInt8) valueS8 = (1, 0, 4, 0, 0, 1, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0)',
                '(vSInt16) valueS16 = (1, 0, 4, 0, 0, 1, 0, 4)',
                '(vSInt32) valueS32 = (4, 3, 2, 1)',
                '(vBool32) valueBool32 = (0, 1, 0, 1)',
            ])



# RUN: %PYTHON %s 2>&1 | FileCheck %s

import ctypes
import sys
from mlir.ir import *
from mlir.dialects import builtin
from mlir.dialects import linalg
from mlir.dialects import std
from mlir.passmanager import *
from mlir.execution_engine import *


# Log everything to stderr and flush so that we have a unified stream to match
# errors/info emitted by MLIR to stderr.
def log(*args):
  print(*args, file=sys.stderr)
  sys.stderr.flush()


matmul_boiler = """
func @main() -> f32 attributes {llvm.emit_c_interface} {
  %v0 = arith.constant 0.0 : f32
  %v1 = arith.constant 1.0 : f32
  %v2 = arith.constant 2.0 : f32

  %A = memref.alloc() : memref<4x16xf32>
  %B = memref.alloc() : memref<16x8xf32>
  %C = memref.alloc() : memref<4x8xf32>
  linalg.fill(%v1, %A) : f32, memref<4x16xf32>
  linalg.fill(%v2, %B) : f32, memref<16x8xf32>
  linalg.fill(%v0, %C) : f32, memref<4x8xf32>

  call @matmul_on_buffers(%A, %B, %C) :
    (memref<4x16xf32>, memref<16x8xf32>, memref<4x8xf32>) -> ()

  %c0 = arith.constant 0 : index
  %0 = memref.load %C[%c0, %c0] : memref<4x8xf32>

  // TODO: FFI-based solution to allow testing and printing with python code.
  return %0 : f32
}
"""

fill_boiler = """
func @main() -> i32 attributes {llvm.emit_c_interface} {
  %O = memref.alloc() : memref<4x16xi32>
  %min = arith.constant -1000.0 : f64
  %max = arith.constant 1000.0 : f64
  %seed = arith.constant 42 : i32

  call @fill_on_buffers(%min, %max, %seed, %O) :
    (f64, f64, i32, memref<4x16xi32>) -> ()

  %c0 = arith.constant 0 : index
  %0 = memref.load %O[%c0, %c0] : memref<4x16xi32>

  // TODO: FFI-based solution to allow testing and printing with python code.
  return %0 : i32
}
"""

conv_boiler = """
func @main() -> i32 attributes {llvm.emit_c_interface} {
  %v0 = arith.constant 0 : i32
  %v1 = arith.constant 1.0 : f64
  %v2 = arith.constant 2.0 : f64

  %input = memref.alloc() : memref<1x4x16x1xf64>
  %filter = memref.alloc() : memref<2x2x1xf64>
  %output = memref.alloc() : memref<1x2x4x1xi32>
  linalg.fill(%v1, %input) : f64, memref<1x4x16x1xf64>
  linalg.fill(%v2, %filter) : f64, memref<2x2x1xf64>
  linalg.fill(%v0, %output) : i32, memref<1x2x4x1xi32>

  call @conv_on_buffers(%input, %filter, %output) :
    (memref<1x4x16x1xf64>, memref<2x2x1xf64>, memref<1x2x4x1xi32>) -> ()

  %c0 = arith.constant 0 : index
  %0 = memref.load %output[%c0, %c0, %c0, %c0] : memref<1x2x4x1xi32>

  // TODO: FFI-based solution to allow testing and printing with python code.
  return %0 : i32
}
"""

pooling_boiler = """
func @main() -> i32 attributes {llvm.emit_c_interface} {
  %v0 = arith.constant 0 : i32
  %v42 = arith.constant 42.0 : f64
  %v77 = arith.constant 77.0 : f64
  %v-13 = arith.constant -13.0 : f64
  %v1 = arith.constant 1.0 : f64

  %input = memref.alloc() : memref<1x4x16x1xf64>
  %shape = memref.alloc() : memref<2x2xf64>
  %output = memref.alloc() : memref<1x2x4x1xi32>
  linalg.fill(%v1, %input) : f64, memref<1x4x16x1xf64>
  linalg.fill(%v1, %shape) : f64, memref<2x2xf64>
  linalg.fill(%v0, %output) : i32, memref<1x2x4x1xi32>

  %c0 = arith.constant 0 : index
  %c1 = arith.constant 1 : index
  %c2 = arith.constant 2 : index
  memref.store %v42, %input[%c0, %c0, %c0, %c0] : memref<1x4x16x1xf64>
  memref.store %v77, %input[%c0, %c0, %c1, %c0] : memref<1x4x16x1xf64>
  memref.store %v-13, %input[%c0, %c0, %c2, %c0] : memref<1x4x16x1xf64>

  call @pooling_on_buffers(%input, %shape, %output) :
    (memref<1x4x16x1xf64>, memref<2x2xf64>, memref<1x2x4x1xi32>) -> ()

  %0 = memref.load %output[%c0, %c0, %c0, %c0] : memref<1x2x4x1xi32>

  // TODO: FFI-based solution to allow testing and printing with python code.
  return %0 : i32
}
"""


def transform(module, boilerplate):
  import mlir.conversions
  import mlir.all_passes_registration
  import mlir.transforms

  # TODO: Allow cloning functions from one module to another.
  # Atm we have to resort to string concatenation.
  mod = Module.parse(
      str(module.operation.regions[0].blocks[0].operations[0].operation) +
      boilerplate)
  pm = PassManager.parse(
      "builtin.func(convert-linalg-to-loops, lower-affine, " +
      "convert-scf-to-std, arith-expand, memref-expand), convert-vector-to-llvm," +
      "convert-memref-to-llvm, convert-std-to-llvm," +
      "reconcile-unrealized-casts")
  pm.run(mod)
  return mod


def test_matmul_builtin():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):

      @builtin.FuncOp.from_py_func(
          MemRefType.get((4, 16), f32), MemRefType.get((16, 8), f32),
          MemRefType.get((4, 8), f32))
      def matmul_on_buffers(lhs, rhs, out):
        linalg.matmul(lhs, rhs, outs=[out])

    execution_engine = ExecutionEngine(transform(module, matmul_boiler))

    # TODO: FFI-based solution to allow testing and printing with python code.
    # Prepare arguments: one result f32.
    # Arguments must be passed as pointers.
    c_float_p = ctypes.c_float * 1
    res = c_float_p(-1.)
    execution_engine.invoke("main", res)

    log("RESULT: ", res[0])
    # CHECK: RESULT: 32.0


test_matmul_builtin()


def test_matmul_generic():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):

      @builtin.FuncOp.from_py_func(
          MemRefType.get((4, 16), f32), MemRefType.get((16, 8), f32),
          MemRefType.get((4, 8), f32))
      def matmul_on_buffers(lhs, rhs, out):
        linalg.matmul(lhs, rhs, outs=[out], emit_generic=True)

    execution_engine = ExecutionEngine(transform(module, matmul_boiler))

    # TODO: FFI-based solution to allow testing and printing with python code.
    # Prepare arguments: one result f32.
    # Arguments must be passed as pointers.
    c_float_p = ctypes.c_float * 1
    res = c_float_p(-1.)
    execution_engine.invoke("main", res)

    log("RESULT: ", res[0])
    # CHECK: RESULT: 32.0


test_matmul_generic()


def test_fill_builtin():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f64 = F64Type.get()
    i32 = IntegerType.get_signless(32)
    with InsertionPoint(module.body):

      @builtin.FuncOp.from_py_func(f64, f64, i32, MemRefType.get((4, 16), i32))
      def fill_on_buffers(min, max, seed, out):
        linalg.fill_rng_2d(min, max, seed, outs=[out])

    execution_engine = ExecutionEngine(transform(module, fill_boiler))

    # TODO: FFI-based solution to allow testing and printing with python code.
    # Prepare arguments: one result i32.
    # Arguments must be passed as pointers.
    c_int_p = ctypes.c_int * 1
    res = c_int_p(-1)
    execution_engine.invoke("main", res)

    log("RESULT: ", res[0])
    # CHECK: RESULT: -480


test_fill_builtin()


def test_fill_generic():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f64 = F64Type.get()
    i32 = IntegerType.get_signless(32)
    with InsertionPoint(module.body):

      @builtin.FuncOp.from_py_func(f64, f64, i32, MemRefType.get((4, 16), i32))
      def fill_on_buffers(min, max, seed, out):
        linalg.fill_rng_2d(min, max, seed, outs=[out], emit_generic=True)

    execution_engine = ExecutionEngine(transform(module, fill_boiler))

    # TODO: FFI-based solution to allow testing and printing with python code.
    # Prepare arguments: one result i32.
    # Arguments must be passed as pointers.
    c_int_p = ctypes.c_int * 1
    res = c_int_p(-1)
    execution_engine.invoke("main", res)

    log("RESULT: ", res[0])
    # CHECK: RESULT: -480


test_fill_generic()


def test_max_pooling_builtin():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f64 = F64Type.get()
    i32 = IntegerType.get_signless(32)
    with InsertionPoint(module.body):

      @builtin.FuncOp.from_py_func(
          MemRefType.get((1, 4, 16, 1), f64), MemRefType.get((2, 2), f64),
          MemRefType.get((1, 2, 4, 1), i32))
      def pooling_on_buffers(input, shape, output):
        linalg.pooling_nhwc_max(
            input, shape, outs=[output], strides=[2, 4], dilations=[1, 2])

    execution_engine = ExecutionEngine(transform(module, pooling_boiler))

    # TODO: FFI-based solution to allow testing and printing with python code.
    # Prepare arguments: one result i32.
    # Arguments must be passed as pointers.
    c_int_p = ctypes.c_int * 1
    res = c_int_p(-1)
    execution_engine.invoke("main", res)

    log("RESULT: ", res[0])
    # 77 is not selected due to the dilation 2 in the second dimension.
    # CHECK: RESULT: 42


test_max_pooling_builtin()


def test_max_pooling_generic():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f64 = F64Type.get()
    i32 = IntegerType.get_signless(32)
    with InsertionPoint(module.body):

      @builtin.FuncOp.from_py_func(
          MemRefType.get((1, 4, 16, 1), f64), MemRefType.get((2, 2), f64),
          MemRefType.get((1, 2, 4, 1), i32))
      def pooling_on_buffers(input, shape, output):
        linalg.pooling_nhwc_max(
            input,
            shape,
            outs=[output],
            strides=[2, 4],
            dilations=[1, 2],
            emit_generic=True)

    execution_engine = ExecutionEngine(transform(module, pooling_boiler))

    # TODO: FFI-based solution to allow testing and printing with python code.
    # Prepare arguments: one result i32.
    # Arguments must be passed as pointers.
    c_int_p = ctypes.c_int * 1
    res = c_int_p(-1)
    execution_engine.invoke("main", res)

    log("RESULT: ", res[0])
    # 77 is not selected due to the dilation 2 in the second dimension.
    # CHECK: RESULT: 42


test_max_pooling_generic()


def test_min_pooling_builtin():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f64 = F64Type.get()
    i32 = IntegerType.get_signless(32)
    with InsertionPoint(module.body):

      @builtin.FuncOp.from_py_func(
          MemRefType.get((1, 4, 16, 1), f64), MemRefType.get((2, 2), f64),
          MemRefType.get((1, 2, 4, 1), i32))
      def pooling_on_buffers(input, shape, output):
        linalg.pooling_nhwc_min(
            input, shape, outs=[output], strides=[2, 4], dilations=[1, 2])

    execution_engine = ExecutionEngine(transform(module, pooling_boiler))

    # TODO: FFI-based solution to allow testing and printing with python code.
    # Prepare arguments: one result i32.
    # Arguments must be passed as pointers.
    c_int_p = ctypes.c_int * 1
    res = c_int_p(-1)
    execution_engine.invoke("main", res)

    log("RESULT: ", res[0])
    # CHECK: RESULT: -13


test_min_pooling_builtin()


def test_min_pooling_generic():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f64 = F64Type.get()
    i32 = IntegerType.get_signless(32)
    with InsertionPoint(module.body):

      @builtin.FuncOp.from_py_func(
          MemRefType.get((1, 4, 16, 1), f64), MemRefType.get((2, 2), f64),
          MemRefType.get((1, 2, 4, 1), i32))
      def pooling_on_buffers(input, shape, output):
        linalg.pooling_nhwc_min(
            input,
            shape,
            outs=[output],
            strides=[2, 4],
            dilations=[1, 2],
            emit_generic=True)

    execution_engine = ExecutionEngine(transform(module, pooling_boiler))

    # TODO: FFI-based solution to allow testing and printing with python code.
    # Prepare arguments: one result i32.
    # Arguments must be passed as pointers.
    c_int_p = ctypes.c_int * 1
    res = c_int_p(-1)
    execution_engine.invoke("main", res)

    log("RESULT: ", res[0])
    # CHECK: RESULT: -13


test_min_pooling_generic()




import unittest2

from lldbsuite.test.decorators import *
from lldbsuite.test.concurrent_base import ConcurrentEventsBase
from lldbsuite.test.lldbtest import TestBase


@skipIfWindows
class ConcurrentBreakpointDelayBreakpointOneSignal(ConcurrentEventsBase):

    mydir = ConcurrentEventsBase.compute_mydir(__file__)

    # Atomic sequences are not supported yet for MIPS in LLDB.
    @skipIf(triple='^mips')
    def test(self):
        """Test two threads that trigger a breakpoint (one with a 1 second delay) and one signal thread. """
        self.build()
        self.do_thread_actions(num_breakpoint_threads=1,
                               num_delay_breakpoint_threads=1,
                               num_signal_threads=1)



"""Test the LLDB module cache funcionality."""

import glob
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import os
import time


class ModuleCacheTestcaseSimple(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number in a(int) to break at.
        self.cache_dir = os.path.join(self.getBuildDir(), 'lldb-module-cache')
        # Set the lldb module cache directory to a directory inside the build
        # artifacts directory so no other tests are interfered with.
        self.runCmd('settings set symbols.lldb-index-cache-path "%s"' % (self.cache_dir))
        self.runCmd('settings set symbols.enable-lldb-index-cache true')
        self.build()


    def get_module_cache_files(self, basename):
        module_file_glob = os.path.join(self.cache_dir,
                "llvmcache-*%s*-symtab-*" % (basename))
        return glob.glob(module_file_glob)

    # Doesn't depend on any specific debug information.
    @no_debug_info_test
    @skipIfWindows
    def test(self):
        """
            Test module cache functionality for a simple object file.

            This will test that if we enable the module cache, we have a
            corresponding index cache entry for the symbol table for the
            executable. It also removes the executable, rebuilds so that the
            modification time of the binary gets updated, and then creates a new
            target and should cause the cache to get updated so the cache file
            should get an updated modification time.
        """
        exe = self.getBuildArtifact("a.out")

        # Create a module with no depedencies.
        target = self.createTestTarget(load_dependent_modules=False)

        # Get the executable module and get the number of symbols to make
        # sure the symbol table gets parsed and cached. The module cache is
        # enabled in the setUp() function.
        main_module = target.GetModuleAtIndex(0)
        self.assertTrue(main_module.IsValid())
        # Make sure the symbol table gets loaded and cached
        main_module.GetNumSymbols()
        cache_files = self.get_module_cache_files("a.out")
        self.assertEqual(len(cache_files), 1,
                         "make sure there is only one cache file for 'a.out'")
        symtab_cache_path = cache_files[0]
        exe_mtime_1 = os.path.getmtime(exe)
        symtab_mtime_1 = os.path.getmtime(symtab_cache_path)
        # Now remove the executable and sleep for a few seconds to make sure we
        # get a different creation and modification time for the file since some
        # OSs store the modification time in seconds since Jan 1, 1970.
        os.remove(exe)
        self.assertEqual(os.path.exists(exe), False,
                         'make sure we were able to remove the executable')
        time.sleep(2)
        # Now rebuild the binary so it has a different content which should
        # update the UUID to make the cache miss when it tries to load the
        # symbol table from the binary at the same path.
        self.build(dictionary={'CFLAGS_EXTRAS': '-DEXTRA_FUNCTION'})
        self.assertEqual(os.path.exists(exe), True,
                         'make sure executable exists after rebuild')
        # Make sure the modification time has changed or this test will fail.
        exe_mtime_2 = os.path.getmtime(exe)
        self.assertNotEqual(
                exe_mtime_1,
                exe_mtime_2,
                "make sure the modification time of the executable has changed")
        # Make sure the module cache still has an out of date cache with the
        # same old modification time.
        self.assertEqual(symtab_mtime_1,
                         os.path.getmtime(symtab_cache_path),
                         "check that the 'symtab' cache file modification time doesn't match the executable modification time after rebuild")
        # Create a new target and get the symbols again, and make sure the cache
        # gets updated for the symbol table cache
        target = self.createTestTarget(load_dependent_modules=False)
        main_module = target.GetModuleAtIndex(0)
        self.assertTrue(main_module.IsValid())
        main_module.GetNumSymbols()
        self.assertEqual(os.path.exists(symtab_cache_path), True,
                         'make sure "symtab" cache files exists after cache is updated')
        symtab_mtime_2 = os.path.getmtime(symtab_cache_path)
        self.assertNotEqual(
                symtab_mtime_1,
                symtab_mtime_2,
                'make sure modification time of "symtab-..." changed')



import lldb


class FooSyntheticProvider:

    def __init__(self, valobj, dict):
        self.valobj = valobj
        self.update()

    def update(self):
        self.adjust_for_architecture()

    def num_children(self):
        return 1

    def get_child_at_index(self, index):
        if index != 0:
            return None
        return self.i_ptr.Dereference()

    def get_child_index(self, name):
        if name == "*i_ptr":
            return 0
        return None

    def adjust_for_architecture(self):
        self.lp64 = (
            self.valobj.GetTarget().GetProcess().GetAddressByteSize() == 8)
        self.is_little = (self.valobj.GetTarget().GetProcess(
        ).GetByteOrder() == lldb.eByteOrderLittle)
        self.pointer_size = self.valobj.GetTarget().GetProcess().GetAddressByteSize()
        self.bar = self.valobj.GetChildMemberWithName('b')
        self.i_ptr = self.bar.GetChildMemberWithName('i_ptr')



"""
Test lldb data formatter subsystem.
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DataFormatterBoolRefPtr(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @skipUnlessDarwin
    def test_boolrefptr_with_run_command(self):
        """Test the formatters we use for BOOL& and BOOL* in Objective-C."""
        self.build()
        self.boolrefptr_data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number('main.mm', '// Set break point at this line.')

    def boolrefptr_data_formatter_commands(self):
        """Test the formatters we use for BOOL& and BOOL* in Objective-C."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.mm", self.line, num_expected_locations=1, loc_exact=True)

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd('type format clear', check=False)
            self.runCmd('type summary clear', check=False)
            self.runCmd('type synth clear', check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        isArm = 'arm' in self.getArchitecture()

        # Now check that we use the right summary for BOOL&
        self.expect('frame variable yes_ref',
                    substrs=['YES'])
        self.expect('frame variable no_ref',
                    substrs=['NO'])
        if not(isArm):
            self.expect('frame variable unset_ref', substrs=['12'])

        # Now check that we use the right summary for BOOL*
        self.expect('frame variable yes_ptr',
                    substrs=['YES'])
        self.expect('frame variable no_ptr',
                    substrs=['NO'])
        if not(isArm):
            self.expect('frame variable unset_ptr', substrs=['12'])

        # Now check that we use the right summary for BOOL
        self.expect('frame variable yes',
                    substrs=['YES'])
        self.expect('frame variable no',
                    substrs=['NO'])
        if not(isArm):
            self.expect('frame variable unset', substrs=['12'])

        # BOOL is bool instead of signed char on ARM.
        converted_YES = "-1" if not isArm else "YES"

        self.expect_expr('myField', result_type="BoolBitFields",
                 result_children=[
                     ValueCheck(name="fieldOne", summary="NO"),
                     ValueCheck(name="fieldTwo", summary=converted_YES),
                     ValueCheck(name="fieldThree", summary="NO"),
                     ValueCheck(name="fieldFour", summary="NO"),
                     ValueCheck(name="fieldFive", summary=converted_YES)
                 ])



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

#  This file contains functions to convert between Memrefs and NumPy arrays and vice-versa.

import numpy as np
import ctypes


def make_nd_memref_descriptor(rank, dtype):
    class MemRefDescriptor(ctypes.Structure):
        """
        Build an empty descriptor for the given rank/dtype, where rank>0.
        """

        _fields_ = [
            ("allocated", ctypes.c_longlong),
            ("aligned", ctypes.POINTER(dtype)),
            ("offset", ctypes.c_longlong),
            ("shape", ctypes.c_longlong * rank),
            ("strides", ctypes.c_longlong * rank),
        ]

    return MemRefDescriptor


def make_zero_d_memref_descriptor(dtype):
    class MemRefDescriptor(ctypes.Structure):
        """
        Build an empty descriptor for the given dtype, where rank=0.
        """

        _fields_ = [
            ("allocated", ctypes.c_longlong),
            ("aligned", ctypes.POINTER(dtype)),
            ("offset", ctypes.c_longlong),
        ]

    return MemRefDescriptor


class UnrankedMemRefDescriptor(ctypes.Structure):
    """ Creates a ctype struct for memref descriptor"""

    _fields_ = [("rank", ctypes.c_longlong), ("descriptor", ctypes.c_void_p)]


def get_ranked_memref_descriptor(nparray):
    """
    Return a ranked memref descriptor for the given numpy array.
    """
    if nparray.ndim == 0:
        x = make_zero_d_memref_descriptor(np.ctypeslib.as_ctypes_type(nparray.dtype))()
        x.allocated = nparray.ctypes.data
        x.aligned = nparray.ctypes.data_as(
            ctypes.POINTER(np.ctypeslib.as_ctypes_type(nparray.dtype))
        )
        x.offset = ctypes.c_longlong(0)
        return x

    x = make_nd_memref_descriptor(
        nparray.ndim, np.ctypeslib.as_ctypes_type(nparray.dtype)
    )()
    x.allocated = nparray.ctypes.data
    x.aligned = nparray.ctypes.data_as(
        ctypes.POINTER(np.ctypeslib.as_ctypes_type(nparray.dtype))
    )
    x.offset = ctypes.c_longlong(0)
    x.shape = nparray.ctypes.shape

    # Numpy uses byte quantities to express strides, MLIR OTOH uses the
    # torch abstraction which specifies strides in terms of elements.
    strides_ctype_t = ctypes.c_longlong * nparray.ndim
    x.strides = strides_ctype_t(*[x // nparray.itemsize for x in nparray.strides])
    return x


def get_unranked_memref_descriptor(nparray):
    """
    Return a generic/unranked memref descriptor for the given numpy array.
    """
    d = UnrankedMemRefDescriptor()
    d.rank = nparray.ndim
    x = get_ranked_memref_descriptor(nparray)
    d.descriptor = ctypes.cast(ctypes.pointer(x), ctypes.c_void_p)
    return d


def unranked_memref_to_numpy(unranked_memref, np_dtype):
    """
    Converts unranked memrefs to numpy arrays.
    """
    descriptor = make_nd_memref_descriptor(
        unranked_memref[0].rank, np.ctypeslib.as_ctypes_type(np_dtype)
    )
    val = ctypes.cast(unranked_memref[0].descriptor, ctypes.POINTER(descriptor))
    np_arr = np.ctypeslib.as_array(val[0].aligned, shape=val[0].shape)
    strided_arr = np.lib.stride_tricks.as_strided(
        np_arr,
        np.ctypeslib.as_array(val[0].shape),
        np.ctypeslib.as_array(val[0].strides) * np_arr.itemsize,
    )
    return strided_arr


def ranked_memref_to_numpy(ranked_memref):
    """
    Converts ranked memrefs to numpy arrays.
    """
    np_arr = np.ctypeslib.as_array(
        ranked_memref[0].aligned, shape=ranked_memref[0].shape
    )
    strided_arr = np.lib.stride_tricks.as_strided(
        np_arr,
        np.ctypeslib.as_array(ranked_memref[0].shape),
        np.ctypeslib.as_array(ranked_memref[0].strides) * np_arr.itemsize,
    )
    return strided_arr



"""
Tests that importing ObjC modules in a non-ObjC target doesn't crash LLDB.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class TestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self,"// break here", lldb.SBFileSpec("main.c"))

        # Import foundation to get some ObjC types.
        self.expect("expr --lang objc -- @import Foundation")
        # Do something with NSString (which requires special handling when
        # preparing to run in the target). The expression most likely can't
        # be prepared to run in the target but it should at least not crash LLDB.
        self.expect('expr --lang objc -- [NSString stringWithFormat:@"%d", 1];',
                    error=True,
                    substrs=["Rewriting an Objective-C constant string requires CFStringCreateWithBytes"])



from __future__ import print_function
import lldb
import sys


class WelcomeCommand(object):

    def __init__(self, debugger, session_dict):
        pass

    def get_short_help(self):
        return "Just a docstring for welcome_impl\nA command that says hello to LLDB users"

    def __call__(self, debugger, args, exe_ctx, result):
        print('Hello ' + args + ', welcome to LLDB', file=result)
        return None


class TargetnameCommand(object):

    def __init__(self, debugger, session_dict):
        pass

    def __call__(self, debugger, args, exe_ctx, result):
        target = debugger.GetSelectedTarget()
        file = target.GetExecutable()
        print('Current target ' + file.GetFilename(), file=result)
        if args == 'fail':
            result.SetError('a test for error in command')

    def get_flags(self):
        return lldb.eCommandRequiresTarget


def print_wait_impl(debugger, args, result, dict):
    result.SetImmediateOutputFile(sys.stdout)
    print('Trying to do long task..', file=result)
    import time
    time.sleep(1)
    print('Still doing long task..', file=result)
    time.sleep(1)
    print('Done; if you saw the delays I am doing OK', file=result)


def check_for_synchro(debugger, args, result, dict):
    if debugger.GetAsync():
        print('I am running async', file=result)
    if debugger.GetAsync() == False:
        print('I am running sync', file=result)


def takes_exe_ctx(debugger, args, exe_ctx, result, dict):
    print(str(exe_ctx.GetTarget()), file=result)



from __future__ import absolute_import

from . import TPunitA
from . import TPunitB


def __lldb_init_module(debugger, *args):
    debugger.HandleCommand(
        "command script add -f thepackage.TPunitA.command TPcommandA")
    debugger.HandleCommand(
        "command script add -f thepackage.TPunitB.command TPcommandB")



"""Test that corefiles with an LC_NOTE "kern ver str" load command is used."""



import os
import re
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestKernVerStrLCNOTE(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @skipIf(debug_info=no_match(["dsym"]), bugnumber="This test is looking explicitly for a dSYM")
    @skipIf(archs=no_match(['x86_64']))
    @skipUnlessDarwin
    def test_lc_note(self):
        self.build()
        self.test_exe = self.getBuildArtifact("a.out")
        self.create_corefile = self.getBuildArtifact("create-empty-corefile")
        self.dsym_for_uuid = self.getBuildArtifact("dsym-for-uuid.sh")
        self.corefile = self.getBuildArtifact("core")

        ## We can hook in our dsym-for-uuid shell script to lldb with this env
        ## var instead of requiring a defaults write.
        os.environ['LLDB_APPLE_DSYMFORUUID_EXECUTABLE'] = self.dsym_for_uuid
        self.addTearDownHook(lambda: os.environ.pop('LLDB_APPLE_DSYMFORUUID_EXECUTABLE', None))

        dwarfdump_uuid_regex = re.compile(
            'UUID: ([-0-9a-fA-F]+) \(([^\(]+)\) .*')
        dwarfdump_cmd_output = subprocess.check_output(
                ('/usr/bin/dwarfdump --uuid "%s"' % self.test_exe), shell=True).decode("utf-8")
        aout_uuid = None
        for line in dwarfdump_cmd_output.splitlines():
            match = dwarfdump_uuid_regex.search(line)
            if match:
                aout_uuid = match.group(1)
        self.assertNotEqual(aout_uuid, None, "Could not get uuid of built a.out")

        ###  Create our dsym-for-uuid shell script which returns self.test_exe
        ###  and its dSYM when given self.test_exe's UUID.
        shell_cmds = [
                '#! /bin/sh',
                'ret=0',
                'echo "<?xml version=\\"1.0\\" encoding=\\"UTF-8\\"?>"',
                'echo "<!DOCTYPE plist PUBLIC \\"-//Apple//DTD PLIST 1.0//EN\\" \\"http://www.apple.com/DTDs/PropertyList-1.0.dtd\\">"',
                'echo "<plist version=\\"1.0\\">"',
                '',
                '# the last argument is probably the uuid',
                'while [ $# -gt 1 ]',
                'do',
                '  shift',
                'done',
                'echo "<dict><key>$1</key><dict>"',
                '',
                'if [ "$1" = "%s" ]' % aout_uuid,
                'then',
                '  echo "<key>DBGArchitecture</key><string>x86_64</string>"',
                '  echo "<key>DBGDSYMPath</key><string>%s.dSYM/Contents/Resources/DWARF/%s</string>"' % (self.test_exe, os.path.basename(self.test_exe)),
                '  echo "<key>DBGSymbolRichExecutable</key><string>%s</string>"' % self.test_exe,
                'else',
                '  echo "<key>DBGError</key><string>not found</string>"',
                '  ret=1',
                'fi',
                'echo "</dict></dict></plist>"',
                'exit $ret'
                ]

        with open(self.dsym_for_uuid, "w") as writer:
            for l in shell_cmds:
                writer.write(l + '\n')

        os.chmod(self.dsym_for_uuid, 0o755)

        ### Create our corefile
        retcode = call(self.create_corefile + " " +  self.corefile + " " + self.test_exe, shell=True)

        ### Now run lldb on the corefile
        ### which will give us a UUID
        ### which we call dsym-for-uuid.sh with
        ### which gives us a binary and dSYM
        ### which lldb should load!


        self.target = self.dbg.CreateTarget('')
        err = lldb.SBError()
        self.process = self.target.LoadCore(self.corefile)
        self.assertEqual(self.process.IsValid(), True)
        if self.TraceOn():
            self.runCmd("image list")
        self.assertEqual(self.target.GetNumModules(), 1)
        fspec = self.target.GetModuleAtIndex(0).GetFileSpec()
        filepath = fspec.GetDirectory() + "/" + fspec.GetFilename()
        self.assertEqual(filepath, self.test_exe)



from lldbsuite.test.lldbinline import CommandParser
from lldbsuite.test.lldbtest import Base
import textwrap


class TestCommandParser(Base):

    mydir = Base.compute_mydir(__file__)

    def test_indentation(self):
        """Test indentation handling"""
        filename = self.getBuildArtifact("test_file.cpp")
        with open(filename, "w") as f:
            f.write(textwrap.dedent("""\
                    int q;
                    int w; //% first break
                    int e;
                    int r; //% second break
                    //% continue second
                    //%   continuing indented
                      //% not indented
                    int t; //% third break
                    """))
        p = CommandParser()
        p.parse_source_files([filename])

        def bkpt(line, cmd):
            return {'file_name': filename, 'line_number': line, 'command': cmd}
        self.assertEqual(
            p.breakpoints, [
                bkpt(2, 'first break'),
                bkpt(4, 'second break\ncontinue second\n  continuing indented\nnot indented'),
                bkpt(8, "third break")])



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class TestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @no_debug_info_test
    def test_file_close_invalid_arg(self):
        self.expect("platform file read y", error=True,
                    substrs=["'y' is not a valid file descriptor."])
        # 'file read' takes an option, so this will be treated as an option.
        self.expect("platform file read -1", error=True,
                    substrs=["unknown or ambiguous option"])



"""
Use lldb Python SBFrame API to get the argument values of the call stacks.
And other SBFrame API tests.
"""

from __future__ import print_function


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FrameAPITestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def test_get_arg_vals_for_call_stack(self):
        """Exercise SBFrame.GetVariables() API to get argument vals."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c by name 'c'.
        breakpoint = target.BreakpointCreateByName('c', 'a.out')
        self.trace("breakpoint:", breakpoint)
        self.assertTrue(breakpoint and
                        breakpoint.GetNumLocations() == 1,
                        VALID_BREAKPOINT)

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())

        process = target.GetProcess()
        self.assertEqual(process.GetState(), lldb.eStateStopped,
                        PROCESS_STOPPED)

        # Keeps track of the number of times 'a' is called where it is within a
        # depth of 3 of the 'c' leaf function.
        callsOfA = 0

        from six import StringIO as SixStringIO
        session = SixStringIO()
        while process.GetState() == lldb.eStateStopped:
            thread = lldbutil.get_stopped_thread(
                process, lldb.eStopReasonBreakpoint)
            self.assertIsNotNone(thread)
            # Inspect at most 3 frames.
            numFrames = min(3, thread.GetNumFrames())
            for i in range(numFrames):
                frame = thread.GetFrameAtIndex(i)
                if self.TraceOn():
                    print("frame:", frame)

                name = frame.GetFunction().GetName()
                if name == 'a':
                    callsOfA = callsOfA + 1

                # We'll inspect only the arguments for the current frame:
                #
                # arguments     => True
                # locals        => False
                # statics       => False
                # in_scope_only => True
                valList = frame.GetVariables(True, False, False, True)
                argList = []
                for val in valList:
                    argList.append("(%s)%s=%s" % (val.GetTypeName(),
                                                  val.GetName(),
                                                  val.GetValue()))
                print("%s(%s)" % (name, ", ".join(argList)), file=session)

                # Also check the generic pc & stack pointer.  We can't test their absolute values,
                # but they should be valid.  Uses get_GPRs() from the lldbutil
                # module.
                gpr_reg_set = lldbutil.get_GPRs(frame)
                pc_value = gpr_reg_set.GetChildMemberWithName("pc")
                self.assertTrue(pc_value, "We should have a valid PC.")
                pc_value_int = int(pc_value.GetValue(), 0)
                # Make sure on arm targets we dont mismatch PC value on the basis of thumb bit.
                # Frame PC will not have thumb bit set in case of a thumb
                # instruction as PC.
                if self.getArchitecture() in ['arm', 'armv7', 'armv7k']:
                    pc_value_int &= ~1
                self.assertEqual(
                    pc_value_int, frame.GetPC(),
                    "PC gotten as a value should equal frame's GetPC")
                sp_value = gpr_reg_set.GetChildMemberWithName("sp")
                self.assertTrue(
                    sp_value, "We should have a valid Stack Pointer.")
                self.assertEqual(
                    int(sp_value.GetValue(), 0), frame.GetSP(),
                    "SP gotten as a value should equal frame's GetSP")

            print("---", file=session)
            process.Continue()

        # At this point, the inferior process should have exited.
        self.assertEqual(
            process.GetState(), lldb.eStateExited,
            PROCESS_EXITED)

        # Expect to find 'a' on the call stacks two times.
        self.assertEqual(callsOfA, 2,
                        "Expect to find 'a' on the call stacks two times")
        # By design, the 'a' call frame has the following arg vals:
        #     o a((int)val=1, (char)ch='A')
        #     o a((int)val=3, (char)ch='A')
        if self.TraceOn():
            print("Full stack traces when stopped on the breakpoint 'c':")
            print(session.getvalue())
        self.expect(session.getvalue(), "Argument values displayed correctly",
                    exe=False,
                    substrs=["a((int)val=1, (char)ch='A')",
                             "a((int)val=3, (char)ch='A')"])

    def test_frame_api_boundary_condition(self):
        """Exercise SBFrame APIs with boundary condition inputs."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c by name 'c'.
        breakpoint = target.BreakpointCreateByName('c', 'a.out')
        self.trace("breakpoint:", breakpoint)
        self.assertTrue(breakpoint and
                        breakpoint.GetNumLocations() == 1,
                        VALID_BREAKPOINT)

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())

        process = target.GetProcess()
        self.assertEqual(process.GetState(), lldb.eStateStopped,
                        PROCESS_STOPPED)

        thread = lldbutil.get_stopped_thread(
            process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)
        frame = thread.GetFrameAtIndex(0)
        if self.TraceOn():
            print("frame:", frame)

        # Boundary condition testings.
        val1 = frame.FindVariable(None, True)
        val2 = frame.FindVariable(None, False)
        val3 = frame.FindValue(None, lldb.eValueTypeVariableGlobal)
        if self.TraceOn():
            print("val1:", val1)
            print("val2:", val2)

        frame.EvaluateExpression(None)

    def test_frame_api_IsEqual(self):
        """Exercise SBFrame API IsEqual."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c by name 'c'.
        breakpoint = target.BreakpointCreateByName('c', 'a.out')
        self.trace("breakpoint:", breakpoint)
        self.assertTrue(breakpoint and
                        breakpoint.GetNumLocations() == 1,
                        VALID_BREAKPOINT)

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())

        process = target.GetProcess()
        self.assertEqual(process.GetState(), lldb.eStateStopped,
                        PROCESS_STOPPED)

        thread = lldbutil.get_stopped_thread(
            process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        frameEntered = thread.GetFrameAtIndex(0)
        if self.TraceOn():
            print(frameEntered)
            lldbutil.print_stacktrace(thread)
        self.assertTrue(frameEntered)

        # Doing two step overs while still inside c().
        thread.StepOver()
        thread.StepOver()
        self.assertTrue(thread)
        frameNow = thread.GetFrameAtIndex(0)
        if self.TraceOn():
            print(frameNow)
            lldbutil.print_stacktrace(thread)
        self.assertTrue(frameNow)

        # The latest two frames are considered equal.
        self.assertTrue(frameEntered.IsEqual(frameNow))

        # Now let's step out of frame c().
        thread.StepOutOfFrame(frameNow)
        frameOutOfC = thread.GetFrameAtIndex(0)
        if self.TraceOn():
            print(frameOutOfC)
            lldbutil.print_stacktrace(thread)
        self.assertTrue(frameOutOfC)

        # The latest two frames should not be equal.
        self.assertFalse(frameOutOfC.IsEqual(frameNow))



#!/usr/bin/env python

"""
A simple bench runner which delegates to the ./dotest.py test driver to run the
benchmarks defined in the list named 'benches'.

You need to hand edit 'benches' to modify/change the command lines passed to the
test driver.

Use the following to get only the benchmark results in your terminal output:

    ./bench.py -e /Volumes/data/lldb/svn/regression/build/Debug/lldb -x '-F Driver::MainLoop()' 2>&1 | grep -P '^lldb.*benchmark:'
"""

from __future__ import print_function
from __future__ import absolute_import

import os
from optparse import OptionParser

# dotest.py invocation with no '-e exe-path' uses lldb as the inferior program,
# unless there is a mentioning of custom executable program.
benches = [
    # Measure startup delays creating a target, setting a breakpoint, and run
    # to breakpoint stop.
    './dotest.py -v +b %E %X -n -p TestStartupDelays.py',

    # Measure 'frame variable' response after stopping at a breakpoint.
    './dotest.py -v +b %E %X -n -p TestFrameVariableResponse.py',

    # Measure stepping speed after stopping at a breakpoint.
    './dotest.py -v +b %E %X -n -p TestSteppingSpeed.py',

    # Measure expression cmd response with a simple custom executable program.
    './dotest.py +b -n -p TestExpressionCmd.py',

    # Attach to a spawned process then run disassembly benchmarks.
    './dotest.py -v +b -n %E -p TestDoAttachThenDisassembly.py'
]


def main():
    """Read the items from 'benches' and run the command line one by one."""
    parser = OptionParser(usage="""\
%prog [options]
Run the standard benchmarks defined in the list named 'benches'.\
""")
    parser.add_option('-e', '--executable',
                      type='string', action='store',
                      dest='exe',
                      help='The target program launched by lldb.')
    parser.add_option('-x', '--breakpoint-spec',
                      type='string', action='store',
                      dest='break_spec',
                      help='The lldb breakpoint spec for the target program.')

    # Parses the options, if any.
    opts, args = parser.parse_args()

    print("Starting bench runner....")

    for item in benches:
        command = item.replace('%E',
                               '-e "%s"' % opts.exe if opts.exe else '')
        command = command.replace('%X', '-x "%s"' %
                                  opts.break_spec if opts.break_spec else '')
        print("Running %s" % (command))
        os.system(command)

    print("Bench runner done.")

if __name__ == '__main__':
    main()



"""
Test that we read the exported symbols from the dyld trie
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class DyldTrieSymbolsTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    NO_DEBUG_INFO_TESTCASE = True

    @skipIfRemote
    @skipUnlessDarwin

    def test_dyld_trie_symbols(self):
        """Test that we make create symbol table entries from the dyld trie data structure."""
        self.build()
        unstripped_exe = self.getBuildArtifact("a.out")
        stripped_exe = self.getBuildArtifact("a.out-stripped")

        unstripped_target = self.dbg.CreateTarget(unstripped_exe)
        self.assertTrue(unstripped_target.IsValid(), "Got a vaid stripped target.")

        # Verify that the expected symbols are present in an unstripped
        # binary, and that we didn't duplicate the entries in the symbol
        # table.
        unstripped_bazval_symbols = unstripped_target.FindSymbols("bazval")
        self.assertEqual(unstripped_bazval_symbols.GetSize(), 1)
        unstripped_patval_symbols = unstripped_target.FindSymbols("patval")
        self.assertEqual(unstripped_patval_symbols.GetSize(), 1)
        unstripped_Z3foo_symbols = unstripped_target.FindSymbols("_Z3foov")
        self.assertEqual(unstripped_Z3foo_symbols.GetSize(), 1)
        unstripped_foo_symbols = unstripped_target.FindSymbols("foo")
        self.assertEqual(unstripped_foo_symbols.GetSize(), 1)

        # make sure we can look up the mangled name, demangled base name,
        # demangled name with argument.
        unstripped_Z3pat_symbols = unstripped_target.FindSymbols("_Z3pati")
        self.assertEqual(unstripped_Z3pat_symbols.GetSize(), 1)
        unstripped_pat_symbols = unstripped_target.FindSymbols("pat")
        self.assertEqual(unstripped_pat_symbols.GetSize(), 1)

        unstripped_bar_symbols = unstripped_target.FindSymbols("bar")
        self.assertEqual(unstripped_bar_symbols.GetSize(), 1)



        # Verify that we can retrieve all the symbols with external
        # linkage after the binary has been stripped; they should not
        # exist in the nlist records at this point and can only be
        # retrieved from the dyld trie structure.

        stripped_target = self.dbg.CreateTarget(stripped_exe)
        self.assertTrue(stripped_target.IsValid(), "Got a vaid stripped target.")

        # Check that we're able to still retrieve all the symbols after
        # the binary has been stripped. Check that one we know will be
        # removed is absent.
        stripped_bazval_symbols = stripped_target.FindSymbols("bazval")
        self.assertEqual(stripped_bazval_symbols.GetSize(), 1)
        stripped_patval_symbols = stripped_target.FindSymbols("patval")
        self.assertEqual(stripped_patval_symbols.GetSize(), 1)
        stripped_Z3foo_symbols = stripped_target.FindSymbols("_Z3foov")
        self.assertEqual(stripped_Z3foo_symbols.GetSize(), 1)
        stripped_foo_symbols = stripped_target.FindSymbols("foo")
        self.assertEqual(stripped_foo_symbols.GetSize(), 1)

        # make sure we can look up the mangled name, demangled base name,
        # demangled name with argument.
        stripped_Z3pat_symbols = stripped_target.FindSymbols("_Z3pati")
        self.assertEqual(stripped_Z3pat_symbols.GetSize(), 1)
        stripped_pat_symbols = stripped_target.FindSymbols("pat")
        self.assertEqual(stripped_pat_symbols.GetSize(), 1)

        # bar should have been strippped.  We should not find it, or the
        # stripping went wrong.
        stripped_bar_symbols = stripped_target.FindSymbols("bar")
        self.assertEqual(stripped_bar_symbols.GetSize(), 0)

        # confirm that we classified objc runtime symbols correctly and
        # stripped off the objc prefix from the symbol names.
        syms_ctx = stripped_target.FindSymbols("SourceBase")
        self.assertEqual(syms_ctx.GetSize(), 2)

        sym1 = syms_ctx.GetContextAtIndex(0).GetSymbol()
        sym2 = syms_ctx.GetContextAtIndex(1).GetSymbol()

        # one of these should be a lldb.eSymbolTypeObjCClass, the other
        # should be lldb.eSymbolTypeObjCMetaClass.
        if sym1.GetType() == lldb.eSymbolTypeObjCMetaClass:
            self.assertEqual(sym2.GetType(), lldb.eSymbolTypeObjCClass)
        else:
            if sym1.GetType() == lldb.eSymbolTypeObjCClass:
                self.assertEqual(sym2.GetType(), lldb.eSymbolTypeObjCMetaClass)
            else:
                self.assertTrue(sym1.GetType() == lldb.eSymbolTypeObjCMetaClass or sym1.GetType() == lldb.eSymbolTypeObjCClass)

        syms_ctx = stripped_target.FindSymbols("SourceDerived._derivedValue")
        self.assertEqual(syms_ctx.GetSize(), 1)
        sym = syms_ctx.GetContextAtIndex(0).GetSymbol()
        self.assertEqual(sym.GetType(), lldb.eSymbolTypeObjCIVar)



# encoding: utf-8
"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

from ObjCDataFormatterTestCase import ObjCDataFormatterTestCase


class ObjCDataFormatterNSBundle(ObjCDataFormatterTestCase):

    def test_nsbundle_with_run_command(self):
        """Test formatters for NSBundle."""
        self.appkit_tester_impl(self.nsbundle_data_formatter_commands, True)

    @skipUnlessDarwin
    def test_nsbundle_with_run_command_no_sonct(self):
        """Test formatters for NSBundle."""
        self.appkit_tester_impl(self.nsbundle_data_formatter_commands, False)

    def nsbundle_data_formatter_commands(self):
        self.expect(
            'frame variable bundle_string bundle_url main_bundle',
            substrs=[
                '(NSBundle *) bundle_string = ',
                ' @"/System/Library/Frameworks/Accelerate.framework"',
                '(NSBundle *) bundle_url = ',
                ' @"/System/Library/Frameworks/Foundation.framework"',
                '(NSBundle *) main_bundle = ', 'data-formatter-objc'
            ])



# RUN: %PYTHON %s | FileCheck %s

import gc
from mlir.ir import *

def run(f):
  print("\nTEST:", f.__name__)
  f()
  gc.collect()
  assert Context._get_live_count() == 0
  return f


# CHECK-LABEL: TEST: testParsePrint
@run
def testParsePrint():
  with Context() as ctx:
    t = Attribute.parse('"hello"')
  assert t.context is ctx
  ctx = None
  gc.collect()
  # CHECK: "hello"
  print(str(t))
  # CHECK: Attribute("hello")
  print(repr(t))


# CHECK-LABEL: TEST: testParseError
# TODO: Hook the diagnostic manager to capture a more meaningful error
# message.
@run
def testParseError():
  with Context():
    try:
      t = Attribute.parse("BAD_ATTR_DOES_NOT_EXIST")
    except ValueError as e:
      # CHECK: Unable to parse attribute: 'BAD_ATTR_DOES_NOT_EXIST'
      print("testParseError:", e)
    else:
      print("Exception not produced")


# CHECK-LABEL: TEST: testAttrEq
@run
def testAttrEq():
  with Context():
    a1 = Attribute.parse('"attr1"')
    a2 = Attribute.parse('"attr2"')
    a3 = Attribute.parse('"attr1"')
    # CHECK: a1 == a1: True
    print("a1 == a1:", a1 == a1)
    # CHECK: a1 == a2: False
    print("a1 == a2:", a1 == a2)
    # CHECK: a1 == a3: True
    print("a1 == a3:", a1 == a3)
    # CHECK: a1 == None: False
    print("a1 == None:", a1 == None)


# CHECK-LABEL: TEST: testAttrHash
@run
def testAttrHash():
  with Context():
    a1 = Attribute.parse('"attr1"')
    a2 = Attribute.parse('"attr2"')
    a3 = Attribute.parse('"attr1"')
    # CHECK: hash(a1) == hash(a3): True
    print("hash(a1) == hash(a3):", a1.__hash__() == a3.__hash__())

    s = set()
    s.add(a1)
    s.add(a2)
    s.add(a3)
    # CHECK: len(s): 2
    print("len(s): ", len(s))


# CHECK-LABEL: TEST: testAttrCast
@run
def testAttrCast():
  with Context():
    a1 = Attribute.parse('"attr1"')
    a2 = Attribute(a1)
    # CHECK: a1 == a2: True
    print("a1 == a2:", a1 == a2)


# CHECK-LABEL: TEST: testAttrIsInstance
@run
def testAttrIsInstance():
  with Context():
    a1 = Attribute.parse("42")
    a2 = Attribute.parse("[42]")
    assert IntegerAttr.isinstance(a1)
    assert not IntegerAttr.isinstance(a2)
    assert not ArrayAttr.isinstance(a1)
    assert ArrayAttr.isinstance(a2)


# CHECK-LABEL: TEST: testAttrEqDoesNotRaise
@run
def testAttrEqDoesNotRaise():
  with Context():
    a1 = Attribute.parse('"attr1"')
    not_an_attr = "foo"
    # CHECK: False
    print(a1 == not_an_attr)
    # CHECK: False
    print(a1 == None)
    # CHECK: True
    print(a1 != None)


# CHECK-LABEL: TEST: testAttrCapsule
@run
def testAttrCapsule():
  with Context() as ctx:
    a1 = Attribute.parse('"attr1"')
  # CHECK: mlir.ir.Attribute._CAPIPtr
  attr_capsule = a1._CAPIPtr
  print(attr_capsule)
  a2 = Attribute._CAPICreate(attr_capsule)
  assert a2 == a1
  assert a2.context is ctx


# CHECK-LABEL: TEST: testStandardAttrCasts
@run
def testStandardAttrCasts():
  with Context():
    a1 = Attribute.parse('"attr1"')
    astr = StringAttr(a1)
    aself = StringAttr(astr)
    # CHECK: Attribute("attr1")
    print(repr(astr))
    try:
      tillegal = StringAttr(Attribute.parse("1.0"))
    except ValueError as e:
      # CHECK: ValueError: Cannot cast attribute to StringAttr (from Attribute(1.000000e+00 : f64))
      print("ValueError:", e)
    else:
      print("Exception not produced")


# CHECK-LABEL: TEST: testAffineMapAttr
@run
def testAffineMapAttr():
  with Context() as ctx:
    d0 = AffineDimExpr.get(0)
    d1 = AffineDimExpr.get(1)
    c2 = AffineConstantExpr.get(2)
    map0 = AffineMap.get(2, 3, [])

    # CHECK: affine_map<(d0, d1)[s0, s1, s2] -> ()>
    attr_built = AffineMapAttr.get(map0)
    print(str(attr_built))

    attr_parsed = Attribute.parse(str(attr_built))
    assert attr_built == attr_parsed


# CHECK-LABEL: TEST: testFloatAttr
@run
def testFloatAttr():
  with Context(), Location.unknown():
    fattr = FloatAttr(Attribute.parse("42.0 : f32"))
    # CHECK: fattr value: 42.0
    print("fattr value:", fattr.value)

    # Test factory methods.
    # CHECK: default_get: 4.200000e+01 : f32
    print("default_get:", FloatAttr.get(
        F32Type.get(), 42.0))
    # CHECK: f32_get: 4.200000e+01 : f32
    print("f32_get:", FloatAttr.get_f32(42.0))
    # CHECK: f64_get: 4.200000e+01 : f64
    print("f64_get:", FloatAttr.get_f64(42.0))
    try:
      fattr_invalid = FloatAttr.get(
          IntegerType.get_signless(32), 42)
    except ValueError as e:
      # CHECK: invalid 'Type(i32)' and expected floating point type.
      print(e)
    else:
      print("Exception not produced")


# CHECK-LABEL: TEST: testIntegerAttr
@run
def testIntegerAttr():
  with Context() as ctx:
    iattr = IntegerAttr(Attribute.parse("42"))
    # CHECK: iattr value: 42
    print("iattr value:", iattr.value)
    # CHECK: iattr type: i64
    print("iattr type:", iattr.type)

    # Test factory methods.
    # CHECK: default_get: 42 : i32
    print("default_get:", IntegerAttr.get(
        IntegerType.get_signless(32), 42))


# CHECK-LABEL: TEST: testBoolAttr
@run
def testBoolAttr():
  with Context() as ctx:
    battr = BoolAttr(Attribute.parse("true"))
    # CHECK: iattr value: True
    print("iattr value:", battr.value)

    # Test factory methods.
    # CHECK: default_get: true
    print("default_get:", BoolAttr.get(True))


# CHECK-LABEL: TEST: testFlatSymbolRefAttr
@run
def testFlatSymbolRefAttr():
  with Context() as ctx:
    sattr = FlatSymbolRefAttr(Attribute.parse('@symbol'))
    # CHECK: symattr value: symbol
    print("symattr value:", sattr.value)

    # Test factory methods.
    # CHECK: default_get: @foobar
    print("default_get:", FlatSymbolRefAttr.get("foobar"))


# CHECK-LABEL: TEST: testStringAttr
@run
def testStringAttr():
  with Context() as ctx:
    sattr = StringAttr(Attribute.parse('"stringattr"'))
    # CHECK: sattr value: stringattr
    print("sattr value:", sattr.value)

    # Test factory methods.
    # CHECK: default_get: "foobar"
    print("default_get:", StringAttr.get("foobar"))
    # CHECK: typed_get: "12345" : i32
    print("typed_get:", StringAttr.get_typed(
        IntegerType.get_signless(32), "12345"))


# CHECK-LABEL: TEST: testNamedAttr
@run
def testNamedAttr():
  with Context():
    a = Attribute.parse('"stringattr"')
    named = a.get_named("foobar")  # Note: under the small object threshold
    # CHECK: attr: "stringattr"
    print("attr:", named.attr)
    # CHECK: name: foobar
    print("name:", named.name)
    # CHECK: named: NamedAttribute(foobar="stringattr")
    print("named:", named)


# CHECK-LABEL: TEST: testDenseIntAttr
@run
def testDenseIntAttr():
  with Context():
    raw = Attribute.parse("dense<[[0,1,2],[3,4,5]]> : vector<2x3xi32>")
    # CHECK: attr: dense<[{{\[}}0, 1, 2], [3, 4, 5]]>
    print("attr:", raw)

    a = DenseIntElementsAttr(raw)
    assert len(a) == 6

    # CHECK: 0 1 2 3 4 5
    for value in a:
      print(value, end=" ")
    print()

    # CHECK: i32
    print(ShapedType(a.type).element_type)

    raw = Attribute.parse("dense<[true,false,true,false]> : vector<4xi1>")
    # CHECK: attr: dense<[true, false, true, false]>
    print("attr:", raw)

    a = DenseIntElementsAttr(raw)
    assert len(a) == 4

    # CHECK: 1 0 1 0
    for value in a:
      print(value, end=" ")
    print()

    # CHECK: i1
    print(ShapedType(a.type).element_type)


# CHECK-LABEL: TEST: testDenseIntAttrGetItem
@run
def testDenseIntAttrGetItem():
  def print_item(attr_asm):
    attr = DenseIntElementsAttr(Attribute.parse(attr_asm))
    dtype = ShapedType(attr.type).element_type
    try:
      item = attr[0]
      print(f"{dtype}:", item)
    except TypeError as e:
      print(f"{dtype}:", e)

  with Context():
    # CHECK: i1: 1
    print_item("dense<true> : tensor<i1>")
    # CHECK: i8: 123
    print_item("dense<123> : tensor<i8>")
    # CHECK: i16: 123
    print_item("dense<123> : tensor<i16>")
    # CHECK: i32: 123
    print_item("dense<123> : tensor<i32>")
    # CHECK: i64: 123
    print_item("dense<123> : tensor<i64>")
    # CHECK: ui8: 123
    print_item("dense<123> : tensor<ui8>")
    # CHECK: ui16: 123
    print_item("dense<123> : tensor<ui16>")
    # CHECK: ui32: 123
    print_item("dense<123> : tensor<ui32>")
    # CHECK: ui64: 123
    print_item("dense<123> : tensor<ui64>")
    # CHECK: si8: -123
    print_item("dense<-123> : tensor<si8>")
    # CHECK: si16: -123
    print_item("dense<-123> : tensor<si16>")
    # CHECK: si32: -123
    print_item("dense<-123> : tensor<si32>")
    # CHECK: si64: -123
    print_item("dense<-123> : tensor<si64>")

    # CHECK: i7: Unsupported integer type
    print_item("dense<123> : tensor<i7>")


# CHECK-LABEL: TEST: testDenseFPAttr
@run
def testDenseFPAttr():
  with Context():
    raw = Attribute.parse("dense<[0.0, 1.0, 2.0, 3.0]> : vector<4xf32>")
    # CHECK: attr: dense<[0.000000e+00, 1.000000e+00, 2.000000e+00, 3.000000e+00]>

    print("attr:", raw)

    a = DenseFPElementsAttr(raw)
    assert len(a) == 4

    # CHECK: 0.0 1.0 2.0 3.0
    for value in a:
      print(value, end=" ")
    print()

    # CHECK: f32
    print(ShapedType(a.type).element_type)


# CHECK-LABEL: TEST: testDictAttr
@run
def testDictAttr():
  with Context():
    dict_attr = {
      'stringattr':  StringAttr.get('string'),
      'integerattr' : IntegerAttr.get(
        IntegerType.get_signless(32), 42)
    }

    a = DictAttr.get(dict_attr)

    # CHECK attr: {integerattr = 42 : i32, stringattr = "string"}
    print("attr:", a)

    assert len(a) == 2

    # CHECK: 42 : i32
    print(a['integerattr'])

    # CHECK: "string"
    print(a['stringattr'])

    # CHECK: True
    print('stringattr' in a)

    # CHECK: False
    print('not_in_dict' in a)

    # Check that exceptions are raised as expected.
    try:
      _ = a['does_not_exist']
    except KeyError:
      pass
    else:
      assert False, "Exception not produced"

    try:
      _ = a[42]
    except IndexError:
      pass
    else:
      assert False, "expected IndexError on accessing an out-of-bounds attribute"

    # CHECK "empty: {}"
    print("empty: ", DictAttr.get())


# CHECK-LABEL: TEST: testTypeAttr
@run
def testTypeAttr():
  with Context():
    raw = Attribute.parse("vector<4xf32>")
    # CHECK: attr: vector<4xf32>
    print("attr:", raw)
    type_attr = TypeAttr(raw)
    # CHECK: f32
    print(ShapedType(type_attr.value).element_type)


# CHECK-LABEL: TEST: testArrayAttr
@run
def testArrayAttr():
  with Context():
    raw = Attribute.parse("[42, true, vector<4xf32>]")
  # CHECK: attr: [42, true, vector<4xf32>]
  print("raw attr:", raw)
  # CHECK: - 42
  # CHECK: - true
  # CHECK: - vector<4xf32>
  for attr in ArrayAttr(raw):
    print("- ", attr)

  with Context():
    intAttr = Attribute.parse("42")
    vecAttr = Attribute.parse("vector<4xf32>")
    boolAttr = BoolAttr.get(True)
    raw = ArrayAttr.get([vecAttr, boolAttr, intAttr])
  # CHECK: attr: [vector<4xf32>, true, 42]
  print("raw attr:", raw)
  # CHECK: - vector<4xf32>
  # CHECK: - true
  # CHECK: - 42
  arr = ArrayAttr(raw)
  for attr in arr:
    print("- ", attr)
  # CHECK: attr[0]: vector<4xf32>
  print("attr[0]:", arr[0])
  # CHECK: attr[1]: true
  print("attr[1]:", arr[1])
  # CHECK: attr[2]: 42
  print("attr[2]:", arr[2])
  try:
    print("attr[3]:", arr[3])
  except IndexError as e:
    # CHECK: Error: ArrayAttribute index out of range
    print("Error: ", e)
  with Context():
    try:
      ArrayAttr.get([None])
    except RuntimeError as e:
      # CHECK: Error: Invalid attribute (None?) when attempting to create an ArrayAttribute
      print("Error: ", e)
    try:
      ArrayAttr.get([42])
    except RuntimeError as e:
      # CHECK: Error: Invalid attribute when attempting to create an ArrayAttribute
      print("Error: ", e)

  with Context():
    array = ArrayAttr.get([StringAttr.get("a"), StringAttr.get("b")])
    array = array + [StringAttr.get("c")]
    # CHECK: concat: ["a", "b", "c"]
    print("concat: ", array)



"""
Test lldb data formatter subsystem.
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

USE_LIBSTDCPP = "USE_LIBSTDCPP"
USE_LIBCPP = "USE_LIBCPP"

class GenericListDataFormatterTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break at for the different tests.
        self.line = line_number('main.cpp', '// Set break point at this line.')
        self.optional_line = line_number(
            'main.cpp', '// Optional break point at this line.')
        self.final_line = line_number(
            'main.cpp', '// Set final break point at this line.')

    def do_test_with_run_command(self, stdlib_type):
        """Test that that file and class static variables display correctly."""
        self.build(dictionary={stdlib_type: "1"})
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=-1)

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd('type format clear', check=False)
            self.runCmd('type summary clear', check=False)
            self.runCmd('type filter clear', check=False)
            self.runCmd('type synth clear', check=False)
            self.runCmd(
                "settings set target.max-children-count 256",
                check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd("frame variable numbers_list --show-types")

        self.runCmd("type format add -f hex int")

        self.expect("frame variable numbers_list --raw", matching=False,
                    substrs=['size=0',
                             '{}'])

        if stdlib_type == USE_LIBSTDCPP:
            self.expect(
                "frame variable &numbers_list._M_impl._M_node --raw",
                matching=False,
                substrs=[
                    'size=0',
                    '{}'])

        self.expect("frame variable numbers_list",
                    substrs=['size=0',
                             '{}'])

        self.expect("p numbers_list",
                    substrs=['size=0',
                             '{}'])

        self.runCmd("n")

        self.expect("frame variable numbers_list",
                    substrs=['size=1',
                             '[0] = ',
                             '0x12345678'])

        self.runCmd("n")
        self.runCmd("n")
        self.runCmd("n")

        self.expect("frame variable numbers_list",
                    substrs=['size=4',
                             '[0] = ',
                             '0x12345678',
                             '[1] =',
                             '0x11223344',
                             '[2] =',
                             '0xbeeffeed',
                             '[3] =',
                             '0x00abba00'])

        self.runCmd("n")
        self.runCmd("n")

        self.expect("frame variable numbers_list",
                    substrs=['size=6',
                             '[0] = ',
                             '0x12345678',
                             '0x11223344',
                             '0xbeeffeed',
                             '0x00abba00',
                             '[4] =',
                             '0x0abcdef0',
                             '[5] =',
                             '0x0cab0cab'])

        self.expect("p numbers_list",
                    substrs=['size=6',
                             '[0] = ',
                             '0x12345678',
                             '0x11223344',
                             '0xbeeffeed',
                             '0x00abba00',
                             '[4] =',
                             '0x0abcdef0',
                             '[5] =',
                             '0x0cab0cab'])

        # check access-by-index
        self.expect("frame variable numbers_list[0]",
                    substrs=['0x12345678'])
        self.expect("frame variable numbers_list[1]",
                    substrs=['0x11223344'])

        # but check that expression does not rely on us
        self.expect("expression numbers_list[0]", matching=False, error=True,
                    substrs=['0x12345678'])

        # check that MightHaveChildren() gets it right
        self.assertTrue(
            self.frame().FindVariable("numbers_list").MightHaveChildren(),
            "numbers_list.MightHaveChildren() says False for non empty!")

        self.runCmd("n")

        self.expect("frame variable numbers_list",
                    substrs=['size=0',
                             '{}'])

        self.runCmd("n")
        self.runCmd("n")
        self.runCmd("n")
        self.runCmd("n")

        self.expect("frame variable numbers_list",
                    substrs=['size=4',
                             '[0] = ', '1',
                             '[1] = ', '2',
                             '[2] = ', '3',
                             '[3] = ', '4'])

        self.runCmd("type format delete int")

        lldbutil.run_break_set_by_file_and_line(self, "main.cpp",
                self.optional_line)
        self.runCmd("continue")

        self.expect("frame variable text_list",
                    substrs=['size=0',
                             '{}'])

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.final_line, num_expected_locations=-1)

        self.runCmd("c", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        self.expect("frame variable text_list",
                    substrs=['size=4',
                             '[0]', 'goofy',
                             '[1]', 'is',
                             '[2]', 'smart',
                             '[3]', '!!!'])

        self.expect("p text_list",
                    substrs=['size=4',
                             '\"goofy\"',
                             '\"is\"',
                             '\"smart\"',
                             '\"!!!\"'])

        # check access-by-index
        self.expect("frame variable text_list[0]",
                    substrs=['goofy'])
        self.expect("frame variable text_list[3]",
                    substrs=['!!!'])

        # but check that expression does not rely on us
        self.expect("expression text_list[0]", matching=False, error=True,
                    substrs=['goofy'])

        # check that MightHaveChildren() gets it right
        self.assertTrue(
            self.frame().FindVariable("text_list").MightHaveChildren(),
            "text_list.MightHaveChildren() says False for non empty!")

    def do_test_ptr_and_ref(self, stdlib_type):
        """Test that ref and ptr to std::list is displayed correctly"""
        self.build(dictionary={stdlib_type: "1"})

        (_, process, _, bkpt) = lldbutil.run_to_source_breakpoint(self,
                'Check ref and ptr',
                lldb.SBFileSpec("main.cpp", False))

        self.expect("frame variable ref",
                    substrs=['size=4',
                             '[0] = ', '1',
                             '[1] = ', '2',
                             '[2] = ', '3',
                             '[3] = ', '4'])


        self.expect("frame variable *ptr",
                    substrs=['size=4',
                             '[0] = ', '1',
                             '[1] = ', '2',
                             '[2] = ', '3',
                             '[3] = ', '4'])

        lldbutil.continue_to_breakpoint(process, bkpt)

        self.expect("frame variable ref",
                    substrs=['size=4',
                             '[0]', 'goofy',
                             '[1]', 'is',
                             '[2]', 'smart',
                             '[3]', '!!!'])

        self.expect("frame variable *ptr",
                    substrs=['size=4',
                             '[0]', 'goofy',
                             '[1]', 'is',
                             '[2]', 'smart',
                             '[3]', '!!!'])
    
    @add_test_categories(["libstdcxx"])
    def test_with_run_command_libstdcpp(self):
        self.do_test_with_run_command(USE_LIBSTDCPP)

    @add_test_categories(["libstdcxx"])
    def test_ptr_and_ref_libstdcpp(self):
        self.do_test_ptr_and_ref(USE_LIBSTDCPP)

    @add_test_categories(["libc++"])
    def test_with_run_command_libcpp(self):
        self.do_test_with_run_command(USE_LIBCPP)

    @add_test_categories(["libc++"])
    def test_ptr_and_ref_libcpp(self):
        self.do_test_ptr_and_ref(USE_LIBCPP)



"""
Test that the user can input a format but it will not prevail over summary format's choices.
"""



import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class UserFormatVSSummaryTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number('main.cpp', '// Set break point at this line.')

    def test_with_run_command(self):
        """Test that the user can input a format but it will not prevail over summary format's choices."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True)

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        self.expect("frame variable p1", substrs=[
                    '(Pair) p1 = (x = 3, y = -3)'])

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd('type format clear', check=False)
            self.runCmd('type summary clear', check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd('type summary add Pair -s "x=${var.x%d},y=${var.y%u}"')

        self.expect("frame variable p1", substrs=[
                    '(Pair) p1 = x=3,y=4294967293'])
        self.expect(
            "frame variable -f x p1",
            substrs=['(Pair) p1 = x=0x00000003,y=0xfffffffd'],
            matching=False)
        self.expect(
            "frame variable -f d p1",
            substrs=['(Pair) p1 = x=3,y=-3'],
            matching=False)
        self.expect("frame variable p1", substrs=[
                    '(Pair) p1 = x=3,y=4294967293'])

        self.runCmd('type summary add Pair -s "x=${var.x%x},y=${var.y%u}"')

        self.expect("frame variable p1", substrs=[
                    '(Pair) p1 = x=0x00000003,y=4294967293'])
        self.expect(
            "frame variable -f d p1",
            substrs=['(Pair) p1 = x=3,y=-3'],
            matching=False)



from lldbsuite.test import lldbinline, lldbplatformutil
from lldbsuite.test import decorators

lldbinline.MakeInlineTest(
    __file__, globals(), [
        decorators.expectedFailureAll(
            bugnumber="llvm.org/PR36715",
            oslist=['windows'])])



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""Base class for all DExTer commands, where a command is a specific Python
function that can be embedded into a comment in the source code under test
which will then be executed by DExTer during debugging.
"""

import abc
from collections import namedtuple
from typing import List

StepExpectInfo = namedtuple('StepExpectInfo', 'expression, path, frame_idx, line_range')

class CommandBase(object, metaclass=abc.ABCMeta):
    def __init__(self):
        self.path = None
        self.lineno = None
        self.raw_text = ''

    def get_label_args(self):
        return list()

    def has_labels(self):
        return False

    @abc.abstractstaticmethod
    def get_name():
        """This abstract method is usually implemented in subclasses as:
        return __class__.__name__
        """

    def get_watches(self) -> List[str]:
        return []

    @abc.abstractmethod
    def eval(self):
        """Evaluate the command.

        This will be called when constructing a Heuristic object to determine
        the debug score.

        Returns:
            The logic for handling the result of CommandBase.eval() must be
            defined in Heuristic.__init__() so a consitent return type between
            commands is not enforced.
        """

    @staticmethod
    def get_subcommands() -> dict:
        """Returns a dictionary of subcommands in the form {name: command} or
        None if no subcommands are required.
        """
        return None



"""
Test lldb data formatter subsystem.
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class PrintObjectArrayTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @skipUnlessDarwin
    def test_print_array(self):
        """Test that expr -O -Z works"""
        self.build()
        self.printarray_data_formatter_commands()

    @skipUnlessDarwin
    def test_print_array_no_const(self):
        """Test that expr -O -Z works"""
        disable_constant_classes = {
            'CC':
            'xcrun clang',  # FIXME: Remove when flags are available upstream.
            'CFLAGS_EXTRAS':
            '-fno-constant-nsnumber-literals ' +
            '-fno-constant-nsarray-literals ' +
            '-fno-constant-nsdictionary-literals'
        }
        self.build(dictionary=disable_constant_classes)
        self.printarray_data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number('main.mm', 'break here')

    def printarray_data_formatter_commands(self):
        """Test that expr -O -Z works"""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.mm", self.line, num_expected_locations=1, loc_exact=True)

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd('type format clear', check=False)
            self.runCmd('type summary clear', check=False)
            self.runCmd('type synth clear', check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.expect(
            'expr --element-count 3 --object-description -- objects',
            substrs=[
                '3735928559',
                '4276993775',
                '3203398366',
                'Hello',
                'World',
                'Two =',
                '1 ='])
        self.expect(
            'poarray 3 objects',
            substrs=[
                '3735928559',
                '4276993775',
                '3203398366',
                'Hello',
                'World',
                'Two =',
                '1 ='])
        self.expect(
            'expr --element-count 3 --object-description --description-verbosity=full -- objects',
            substrs=[
                '[0] =',
                '3735928559',
                '4276993775',
                '3203398366',
                '[1] =',
                'Hello',
                'World',
                '[2] =',
                'Two =',
                '1 ='])
        self.expect(
            'parray 3 objects',
            substrs=[
                '[0] = 0x',
                '[1] = 0x',
                '[2] = 0x'])
        self.expect(
            'expr --element-count 3 -d run -- objects',
            substrs=[
                '3 elements',
                '2 elements',
                '2 key/value pairs'])
        self.expect(
            'expr --element-count 3 -d run --ptr-depth=1 -- objects',
            substrs=[
                '3 elements',
                '3735928559',
                '4276993775',
                '3203398366',
                '2 elements',
                '"Hello"',
                '"World"',
                '2 key/value pairs',
            ])



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class StaticInitializers(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @expectedFailureAll(archs="aarch64", oslist=["freebsd", "linux"],
                        bugnumber="https://bugs.llvm.org/show_bug.cgi?id=44053")
    def test(self):
        """ Test a static initializer. """
        self.build()

        lldbutil.run_to_source_breakpoint(self, '// break here',
                lldb.SBFileSpec("main.cpp", False))

        # We use counter to observe if the initializer was called.
        self.expect_expr("counter", result_type="int", result_value="0")
        self.expect("expr -p -- struct Foo { Foo() { inc_counter(); } }; Foo f;")
        self.expect_expr("counter", result_type="int", result_value="1")

    def test_failing_init(self):
        """ Test a static initializer that fails to execute. """
        self.build()

        lldbutil.run_to_source_breakpoint(self, '// break here',
                lldb.SBFileSpec("main.cpp", False))

        # FIXME: This error message is not even remotely helpful.
        self.expect("expr -p -- struct Foo2 { Foo2() { do_abort(); } }; Foo2 f;", error=True,
                    substrs=["error: couldn't run static initializer:"])

    def test_without_process(self):
        """ Test a static initializer without a running process. """
        self.expect("expr -p -- int i = 0; struct Foo3 { Foo3() { ++i; } }; Foo3 f;", error=True,
                    substrs=["Top-level code needs to be inserted into a runnable target"])



from lldbsuite.test import decorators
from lldbsuite.test import lldbinline

lldbinline.MakeInlineTest(
    __file__, globals(), [
        decorators.add_test_categories(["objc"]),
        decorators.expectedFailureAll(
            oslist=['macosx'], archs=['i386'],
            bugnumber='rdar://28656677')])



"""
Test loading of a kext binary.
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LoadKextTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    mydir = TestBase.compute_mydir(__file__)

    def test_load_kext(self):
        """Test that lldb can load a kext binary."""

        # Create kext from YAML.
        self.yaml2obj("mykext.yaml", self.getBuildArtifact("mykext"))

        target = self.dbg.CreateTarget(self.getBuildArtifact("mykext"))

        self.assertTrue(target.IsValid())

        self.assertEqual(target.GetNumModules(), 1)
        mod = target.GetModuleAtIndex(0)
        self.assertEqual(mod.GetFileSpec().GetFilename(), "mykext")
